@@ -4,28 +4,27 @@ from genericpath import exists
 from logging import getLogger
 from os import makedirs
 from os.path import join
-from re import sub
-from typing import Mapping
 
-from ..global_ import set_global_state
-from ..lib.utils import clean_path, parse_bool
-from ..names import UNDEF
+from ..common.args import mm, m, no, add_server_args, NamespaceWithVariables, color, add_data_source_args
+from ..common.names import *
+from ..common.names import UNDEF, COLOR, OFF, VERSION, USER
+from ..lib.utils import parse_bool
 
 log = getLogger(__name__)
 
 # this can be modified during development.  it will be reset from setup.py on release.
-CH2_VERSION = '0.34.2'
+CH2_VERSION = '0.39.1'
 # new database on minor releases.  not sure this will always be a good idea.  we will see.
 DB_VERSION = '-'.join(CH2_VERSION.split('.')[:2])
-DB_EXTN = '.db'   # used to use .sql but auto-complete for sqlite3 didn't work
+
+URI_DEFAULT = 'postgresql://{user}:{passwd}@{db-bind}:{db-port}/activity-{version}'
+URI_PREVIOUS = 'postgresql://{user}:{passwd}@{db-bind}:{db-port}/activity-{version}?search_path={user}:previous'
 
 PROGNAME = 'ch2'
-COMMAND = 'command'
 
 WEB_PORT = 8000
 JUPYTER_PORT = 8001
 
-CALCULATE = 'calculate'
 CONSTANTS = 'constants'
 DATABASE = 'database'
 DIARY = 'diary'
@@ -40,14 +39,13 @@ KIT = 'kit'
 LOAD = 'load'
 NO_OP = 'no-op'
 PACKAGE_FIT_PROFILE = 'package-fit-profile'
-READ = 'read'
 SEARCH = 'search'
 SHOW_SCHEDULE = 'show-schedule'
+SPARKLINE = 'sparkline'
 TEXT = 'text'
 THUMBNAIL = 'thumbnail'
 UNLOCK = 'unlock'
 VALIDATE = 'validate'
-WEB = 'web'
 
 A = 'a'
 ACTIVITY = 'activity'
@@ -66,33 +64,35 @@ ALL = 'all'
 ALL_MESSAGES = 'all-messages'
 ALL_FIELDS = 'all-fields'
 ARG = 'arg'
-BASE = 'base'
+BACKUP = 'backup'
 BATCH = 'batch'
-BIND = 'bind'
 BORDER = 'border'
 CHANGE = 'change'
 CHECK = 'check'
 CMD = 'cmd'
-COLOR = 'color'
 COMPACT = 'compact'
 COMPONENT = 'component'
 CONSTRAINT = 'constraint'
 CONTEXT = 'context'
+CPROFILE = 'cprofile'
 CSV = 'csv'
 D = 'd'
 DARK = 'dark'
 DATA = 'data'
+DATABASES = 'databases'
 DATE = 'date'
+DB = 'db'
 DEFAULT = 'default'
-DEFINE = 'define'
 DELETE = 'delete'
 DESCRIBE = 'describe'
 DESCRIPTION = 'description'
 DEV = 'dev'
 DIR = 'dir'
+DISPLAY = 'display'
 DISABLE = 'disable'
 DISCARD = 'discard'
 DROP = 'drop'
+DURATION = 'duration'
 EMPTY = 'empty'
 ENABLE = 'enable'
 ENGINE = 'engine'
@@ -111,7 +111,9 @@ GREP = 'grep'
 GROUP = 'group'
 HEADER_SIZE = 'header-size'
 HEIGHT = 'height'
+IMAGE_DIR = 'image-dir'
 INTERNAL = 'internal'
+INVERT = 'invert'
 ITEM = 'item'
 K = 'k'
 KARG = 'karg'
@@ -123,6 +125,7 @@ LIMIT_BYTES = 'limit-bytes'
 LIMIT_RECORDS = 'limit-records'
 LOG = 'log'
 LOGS = 'logs'
+LOG_DIR = 'log-dir'
 LONGITUDE = 'longitude'
 LIST = 'list'
 M, MESSAGE = 'm', 'message'
@@ -147,21 +150,20 @@ NAMES = 'names'
 NEW = 'new'
 NOT = 'not'
 NOTEBOOKS = 'notebooks'
+NOTEBOOK_DIR = 'notebook-dir'
 O, OUTPUT = 'o', 'output'
-OFF = 'off'
 OWNER = 'owner'
-PASS = 'pass'
 PATH = 'path'
-P, PATTERN = 'p', 'pattern'
+PATTERN = 'pattern'
 PERMANENT = 'permanent'
-POSTGRESQL = 'postgresql'
 PLAN = 'plan'
-PORT = 'port'
+PREVIOUS = 'previous'
 PRINT = 'print'
+PROCESS = 'process'
 PROFILE = 'profile'
+PROFILES = 'profiles'
 PROFILE_VERSION = 'profile-version'
 PROTOCOL_VERSION = 'protocol-version'
-PROXY = 'proxy'
 PWD = 'pwd'
 QUERY = 'query'
 RAW = 'raw'
@@ -177,35 +179,37 @@ SEGMENTS = 'segments'
 SERVICE = 'service'
 SET = 'set'
 SCHEDULE = 'schedule'
+SCHEMA = 'schema'
+SCHEMAS = 'schemas'
 SECURE = 'secure'
+SECTOR = 'sector'
+SECTORS = 'sectors'
 SHOW = 'show'
 SINGLE = 'single'
 SLICES = 'slices'
 SOURCE = 'source'
 SOURCES = 'sources'
 SOURCE_ID = 'source-id'
-SQLITE = 'sqlite'
 START = 'start'
+STATISTIC = 'statistic'
 STATISTICS = 'statistics'
 STATISTIC_NAMES = 'statistic-names'
 STATISTIC_JOURNALS = 'statistic-journals'
 STATUS = 'status'
 STOP = 'stop'
 SUB_COMMAND = 'sub-command'
+SUB2_COMMAND = 'sub2-command'
 SYSTEM = 'system'
 TABLE = 'table'
 TABLES = 'tables'
 TOKENS = 'tokens'
 TOPIC = 'topic'
 UNDO = 'undo'
-UNLIKE = 'unlike'
 UNSAFE = 'unsafe'
 UNSET = 'unset'
-URI = 'uri'
-USER = 'user'
-V, VERBOSITY = 'v', 'verbosity'
+UPLOAD = 'upload'
+USERS = 'users'
 VALUE = 'value'
-VERSION = 'version'
 W, WARN = 'w', 'warn'
 WAYPOINTS = 'waypoints'
 WIDTH = 'width'
@@ -214,36 +218,9 @@ YEAR = 'year'
 Y = 'y'
 
 
-def mm(name): return '--' + name
-def m(name): return '-' + name
-def no(name): return 'no-%s' % name
-
-
-class NamespaceWithVariables(Mapping):
-
-    def __init__(self, ns):
-        self._dict = vars(ns)
-
-    def __getitem__(self, name):
-        try:
-            value = self._dict[name]
-        except KeyError:
-            value = self._dict[sub('-', '_', name)]
-        return value
-
-    def system_path(self, subdir=None, file=None, version=DB_VERSION, create=True):
-        return base_system_path(self[BASE], subdir=subdir, file=file, version=version, create=create)
-
-    def __iter__(self):
-        return iter(self._dict)
-
-    def __len__(self):
-        return len(self.__dict__)
-
-
 def base_system_path(base, subdir=None, file=None, version=DB_VERSION, create=True):
 
-    from ..lib.utils import clean_path
+    from ..common.io import clean_path
 
     dir = base
     if version: dir = join(dir, version)
@@ -256,94 +233,98 @@ def base_system_path(base, subdir=None, file=None, version=DB_VERSION, create=Tr
         return dir
 
 
-def color(color):
-    if color.lower() not in (LIGHT, DARK, OFF):
-        raise Exception(f'Bad color: {color} ({LIGHT}|{DARK}|{OFF})')
-    return color
-
-
 def make_parser(with_noop=False):
 
-    from ..lib import to_date, to_time
+    from ..lib import to_time
+    from ..common.io import clean_path
 
     parser = ArgumentParser(prog=PROGNAME)
 
-    parser.add_argument(mm(BASE), default='~/.ch2', type=clean_path, metavar='DIR',
-                        help='the base directory for data (default ~/.ch2)')
-    parser.add_argument(mm(READ_ONLY), action='store_true',
-                        help='read-only database (so errors on write)')
-    parser.add_argument(mm(LOG), metavar='FILE',
-                        help='the file name for the log (command name by default)')
-    parser.add_argument(mm(COLOR), type=color,
-                        help=f'pretty stdout log - {LIGHT}|{DARK}|{OFF} (CAPS to save)')
-    parser.add_argument(m(V), mm(VERBOSITY), default=UNDEF, type=int, metavar='N',
-                        help='output level for stderr (0: silent; 5:noisy)')
     parser.add_argument(mm(DEV), action='store_true',
                         help='verbose log and stack trace on error')
+    parser.add_argument(mm(LOG), metavar='FILE',
+                        help='the file name for the log (command name by default)')
+    parser.add_argument(mm(LOG_DIR), metavar='DIR', default='{base}/{version}/logs',
+                        help='the directory for the log')
+    parser.add_argument(mm(COLOR), mm(COLOUR), type=color, dest=COLOR, default=DARK,
+                        help=f'pretty stdout log - {LIGHT}|{DARK}|{OFF}')
+    parser.add_argument(m(V), mm(VERBOSITY), default=UNDEF, type=int, metavar='N',
+                        help='output level for stderr (0: silent; 5:noisy)')
     parser.add_argument(m(V.upper()), mm(VERSION), action='version', version=CH2_VERSION,
                         help='display version and exit')
+    add_server_args(parser, DB, default_port=5432, name='database')
+    add_data_source_args(parser, URI_DEFAULT)
+    parser.add_argument(mm(BASE), default='~/.ch2', metavar='DIR', type=clean_path,
+                        help='the base directory for data (default ~/.ch2)')
+    parser.add_argument(mm(DATA), metavar='DIR', default='{base}/permanent',
+                        help='the root directory for storing data on disk')
+    parser.add_argument(mm(CPROFILE), metavar='DIR', nargs='?', action='append',
+                        help='file for profiling data (development)')
 
-    subparsers = parser.add_subparsers(title='commands', dest=COMMAND)
+    commands = parser.add_subparsers(title='commands', dest=COMMAND)
 
-    # high-level commands used daily
-
-    help = subparsers.add_parser(HELP, help='display help',
+    help = commands.add_parser(HELP, help='display help',
                                  description='display additional help (beyond -h) for any command')
     help.add_argument(TOPIC, nargs='?', metavar=TOPIC, help='the subject for help')
 
-    def add_uri_options(parser, required):
-        database_uri = parser.add_mutually_exclusive_group(required=required)
-        database_uri.add_argument(mm(SQLITE), dest=URI, action='store_const', const=SQLITE,
-                                  help='use an sqlite database with standard parameters')
-        database_uri.add_argument(mm(POSTGRESQL), dest=URI, action='store_const', const=POSTGRESQL,
-                                  help='use a postgresql database with standard parameters')
-        database_uri.add_argument(mm(URI), help='use the given database URI')
-
-    web = subparsers.add_parser(WEB, help='the web interface (probably all you need)',
+    web = commands.add_parser(WEB, help='the web interface (probably all you need)',
                                 description='start, stop and manage the web interface')
     web_cmds = web.add_subparsers(title='sub-commands', dest=SUB_COMMAND, required=True)
 
-    def add_web_server_args(cmd, prefix='', default_address='localhost', default_port=WEB_PORT):
-        if prefix: prefix += '-'
-        cmd.add_argument(mm(prefix + BIND), default='localhost', metavar='ADDRESS',
-                         help='bind address' + f' (default {default_address})' if default_address else '')
-        cmd.add_argument(mm(prefix + PORT), default=default_port, type=int, metavar='PORT',
-                         help=f'port' + f' (default {default_port})' if default_port else '')
-
     def add_warning_args(cmd):
         prefix = WARN + '-'
         cmd.add_argument(mm(prefix + DATA), action='store_true', help='warn user that data may be lost')
         cmd.add_argument(mm(prefix + SECURE), action='store_true', help='warn user that the system is insecure')
 
+    def add_image_dir(cmd):
+        cmd.add_argument(mm(IMAGE_DIR), metavar='DIR', default='{base}/{version}/image',
+                         help='image cache')
+
+    def add_notebook_dir(cmd):
+        cmd.add_argument(mm(NOTEBOOK_DIR), metavar='DIR', default='{base}/{version}/notebook',
+                         help='notebook cache')
+
+    def add_jupyter(cmd):
+        cmd.add_argument(mm(JUPYTER), metavar='URL', default=f'http://localhost:{JUPYTER_PORT}/tree',
+                         help='jupyter URL prefix')
+
     web_start = web_cmds.add_parser(START, help='start the web server', description='start the web server')
-    add_web_server_args(web_start, prefix=WEB)
-    add_web_server_args(web_start, prefix=JUPYTER, default_port=JUPYTER_PORT)
-    add_web_server_args(web_start, prefix=PROXY, default_port=None, default_address=None)
+    add_server_args(web_start, prefix=WEB, default_port=WEB_PORT, name='web server')
+    add_jupyter(web_start)
     add_warning_args(web_start)
+    add_image_dir(web_start)
+    add_notebook_dir(web_start)
     web_cmds.add_parser(STOP, help='stop the web server', description='stop the web server')
     web_cmds.add_parser(STATUS, help='display status of web server', description='display status of web server')
     web_service = web_cmds.add_parser(SERVICE, help='internal use only - use start/stop',
                                       description='internal use only - use start/stop')
-    add_web_server_args(web_service, prefix=WEB)
-    add_web_server_args(web_service, prefix=JUPYTER, default_port=JUPYTER_PORT)
-    add_web_server_args(web_service, prefix=PROXY, default_port=None, default_address=None)
+    add_server_args(web_service, prefix=WEB, default_port=WEB_PORT, name='web server')
+    add_jupyter(web_service)
     add_warning_args(web_service)
-    add_uri_options(web_service, False)
+    add_image_dir(web_service)
+    add_notebook_dir(web_service)
 
-    read = subparsers.add_parser(READ, help='read data (also calls calculate)',
-                                 description='read data (copy it to the permanent store) and '
-                                             'calculate statistics (add information to the database)')
-    read.add_argument(mm(FORCE), action='store_true', help='reprocess existing data')
-    read.add_argument(mm(KIT), m(K), action='append', default=[], metavar='ITEM',
-                      help='kit items associated with activities')
-    read.add_argument(PATH, metavar='PATH', nargs='*', default=[], help='path to FIT file(s) containing data')
-    read.add_argument(mm(KARG), m(K.upper()), action='append', default=[], metavar='NAME=VALUE',
-                      help='keyword argument to be passed to the pipelines (can be repeated)')
-    read.add_argument(mm(WORKER), metavar='ID', type=int, help='internal use only (identifies sub-process workers)')
-    read.add_argument(mm(DISABLE), action='store_true', help='disable following options (they enable by default)')
-    read.add_argument(mm(ACTIVITIES), action='store_true', help='enable (or disable) processing of activity data')
-    read.add_argument(mm(MONITOR), action='store_true', help='enable (or disable) processing of monitor data')
-    read.add_argument(mm(CALCULATE), action='store_true', help='enable (or disable) calculating statistics')
+    upload = commands.add_parser(UPLOAD, help='upload data (copy FIT files to permanent store)',
+                                 description='copy files to the permanent store and (optionally) process the data')
+    upload.add_argument(mm(KARG), m(K.upper()), action='append', default=[], metavar='NAME=VALUE',
+                        help='keyword argument to be passed to the pipelines (can be repeated)')
+    upload.add_argument(mm(KIT), m(K), action='append', default=[], metavar='ITEM',
+                        help='kit items associated with activities')
+    upload.add_argument(mm(no(PROCESS)), action='store_false', dest=PROCESS,
+                        help='do not call process after uploading')
+    upload.add_argument(PATH, metavar='PATH', nargs='*', default=[], help='path to FIT file(s) containing data')
+
+    process = commands.add_parser(PROCESS, help='process data (add information to the database)',
+                                  description='read new files from the permanent store and calculate statistics')
+    process.add_argument(mm(KARG), m(K.upper()), action='append', default=[], metavar='NAME=VALUE',
+                         help='keyword argument to be passed to the pipelines (can be repeated)')
+    # cannot be expressed in argparse (but checked in command) - like and worker are mutually exclusive
+    process.add_argument(mm(LIKE), action='append', default=[], metavar='PATTERN',
+                         help='run only matching pipeline classes')
+    process.add_argument(mm(WORKER), metavar='ID', type=int,
+                         help='internal use only (identifies sub-process workers)')
+    process.add_argument(ARG, nargs='*', metavar='WORKER_ARG',
+                         help=f'internal use only (tasks for {mm(WORKER)})')
 
     def add_search_query(cmd, query_help='search terms (similar to SQL)'):
         cmd.add_argument(QUERY, metavar='QUERY', default=[], nargs='+', help=query_help)
@@ -351,7 +332,7 @@ def make_parser(with_noop=False):
                          help='show value from matching entries')
         cmd.add_argument(mm(SET), metavar='NAME=VALUE', help='update matching entries')
 
-    search = subparsers.add_parser(SEARCH, help='search the database', description='search the database')
+    search = commands.add_parser(SEARCH, help='search the database', description='search the database')
     search_cmds = search.add_subparsers(title='search target', dest=SUB_COMMAND, required=True)
     search_text = search_cmds.add_parser(TEXT, help='search for text in activities',
                                          description='search for text in activities')
@@ -363,7 +344,7 @@ def make_parser(with_noop=False):
                                             description='search for sources')
     add_search_query(search_sources)
 
-    constants = subparsers.add_parser(CONSTANTS, help='set and examine constants',
+    constants = commands.add_parser(CONSTANTS, help='set and examine constants',
                                       description='set and examine constants')
     constants_cmds = constants.add_subparsers(title='sub-commands', dest=SUB_COMMAND, required=True)
     constants_list = constants_cmds.add_parser(LIST, help='list all constant names',
@@ -397,34 +378,11 @@ def make_parser(with_noop=False):
     constants_remove.add_argument(NAME, metavar='NAME', help='name')
     constants_remove.add_argument(mm(FORCE), action='store_true', help='allow remove of multiple constants')
 
-    validate = subparsers.add_parser(VALIDATE, help='check (and optionally fix) data in the database',
+    validate = commands.add_parser(VALIDATE, help='check (and optionally fix) data in the database',
                                      description='check (and optionally fix) data in the database')
     validate.add_argument(mm(FIX), action='store_true', help='correct errors when possible')
 
-    jupyter = subparsers.add_parser(JUPYTER, help='data analysis in jupyter',
-                                    description='data analysis in jupyter')
-    jupyter_cmds = jupyter.add_subparsers(title='sub-commands', dest=SUB_COMMAND, required=True)
-    jupyter_cmds.add_parser(LIST, help='list available templates',
-                            description='list available templates and parameters for jupyter')
-    jupyter_show = jupyter_cmds.add_parser(SHOW, help='display a template (starting server if needed)',
-                                           description='display a template in jupyter (starting server if needed)')
-    jupyter_show.add_argument(NAME, metavar='NAME', help='the template name')
-    jupyter_show.add_argument(ARG, nargs='*', metavar='PARAM', help='template arguments')
-    add_web_server_args(jupyter_show, prefix=JUPYTER, default_port=JUPYTER_PORT)
-    add_web_server_args(jupyter_show, prefix=PROXY, default_port=None, default_address=None)
-    jupyter_start = jupyter_cmds.add_parser(START, help='start a background service',
-                                            description='start jupyter as a background service')
-    add_web_server_args(jupyter_start, prefix=JUPYTER, default_port=JUPYTER_PORT)
-    add_web_server_args(jupyter_start, prefix=PROXY, default_port=None, default_address=None)
-    jupyter_cmds.add_parser(STOP, help='stop the background service',
-                            description='stop the jupyter background service')
-    jupyter_cmds.add_parser(STATUS, help='display status of background service')
-    jupyter_service = jupyter_cmds.add_parser(SERVICE, help='internal use only - use start/stop',
-                                              description='internal use only - use start/stop')
-    add_web_server_args(jupyter_service, prefix=JUPYTER, default_port=JUPYTER_PORT)
-    add_web_server_args(jupyter_service, prefix=PROXY, default_port=None, default_address=None)
-
-    kit = subparsers.add_parser(KIT, help='manage kit',
+    kit = commands.add_parser(KIT, help='manage kit',
                                 description='add, remove, modify and display kit details')
     kit_cmds = kit.add_subparsers(title='sub-commands', dest=SUB_COMMAND, required=True)
     kit_start = kit_cmds.add_parser(START, help='define a new item (new bike, new shoe)',
@@ -469,61 +427,52 @@ def make_parser(with_noop=False):
     kit_dump = kit_cmds.add_parser(DUMP, help='dump to script')
     kit_dump.add_argument(mm(CMD), help='command to use instead of ch2')
 
-    database = subparsers.add_parser(DATABASE, help='configure the database')
-    database_cmds = database.add_subparsers(title='sub-commands', dest=SUB_COMMAND, required=True)
-    database_show = database_cmds.add_parser(SHOW, help='show current configuration')
-    database_list = database_cmds.add_parser(LIST, help='list available profiles')
-    database_load = database_cmds.add_parser(LOAD, help='configure using the given profile')
-    database_load.add_argument(mm(FORCE), action='store_true', help='overwrite existing database')
-    add_uri_options(database_load, True)
-    database_profiles = database_load.add_subparsers(title='profile', dest=PROFILE, required=True)
-    from ..config.utils import profiles
-    for name in profiles():
-        database_profile = database_profiles.add_parser(name)
-        database_profile.add_argument(mm(no(DIARY)), action='store_true', help='skip diary creation (for migration)')
-    database_delete = database_cmds.add_parser(DELETE, help='delete the current database (or one at a given URI)')
-    database_engines = database_delete.add_mutually_exclusive_group(required=False)
-    database_engines.add_argument(mm(SQLITE), dest=URI, action='store_const', const=SQLITE,
-                                  help='use an sqlite database with standard parameters')
-    database_engines.add_argument(mm(POSTGRESQL), dest=URI, action='store_const', const=POSTGRESQL,
-                                  help='use a postgresql database with standard parameters')
-    database_engines.add_argument(mm(URI), help='use the given database URI')
+    db = commands.add_parser(DB, help='configure the database')
+    db_cmds = db.add_subparsers(title='sub-commands', dest=SUB_COMMAND, required=True)
+    db_list = db_cmds.add_parser(LIST, help='show current configuration')
+    db_list_item = db_list.add_subparsers(title='item to list', dest=ITEM, required=True)
+    db_list_item.add_parser(USERS, help='show configured users')
+    db_list_item.add_parser(SCHEMAS, help='show configured schema')
+    db_list_item.add_parser(PROFILES, help='show available profiles')
+    db_list_item.add_parser(DATABASES, help='show configured databases')
+    db_add = db_cmds.add_parser(ADD, help='extend current configuration')
+    db_add_item = db_add.add_subparsers(title='item to add', dest=ITEM, required=True)
+    db_add_item.add_parser(USER, help='add a user (once per cluster)')
+    db_add_item.add_parser(DATABASE, help='add a database (for each version in a cluster)')
+    db_add_profile = db_add_item.add_parser(PROFILE, help='add a profile (for each user and version)')
+    db_add_profile_profiles = db_add_profile.add_subparsers(title='profile', dest=PROFILE, required=True)
+    from ..config.profile import get_profiles
+    for name in get_profiles():
+        db_add_profile_profiles.add_parser(name)
+    db_backup = db_cmds.add_parser(BACKUP, help='backup current configuration')
+    db_backup_item = db_backup.add_subparsers(title='item to backup', dest=ITEM, required=True)
+    db_backup_item.add_parser(SCHEMA, help='backup a schema')
+    db_remove = db_cmds.add_parser(REMOVE, help='reduce current configuration')
+    db_remove_item = db_remove.add_subparsers(title='item to remove', dest=ITEM, required=True)
+    db_remove_item.add_parser(USER, help='remove a user')
+    db_remove_item.add_parser(DATABASE, help='remove a database')
+    db_remove_schema = db_remove_item.add_parser(SCHEMA, help='remove a schema')
+    db_remove_schema.add_argument(mm(no(PREVIOUS)), dest=PREVIOUS, action='store_false',
+                                  help='do not create a :previous copy')
 
-    import_ = subparsers.add_parser(IMPORT, help='import data from a previous version')
-    import_.add_argument(SOURCE, help='version or uri to import')
-    import_engines = import_.add_mutually_exclusive_group(required=False)
-    import_engines.add_argument(mm(SQLITE), dest=ENGINE, action='store_const', const=SQLITE,
-                                help='version is an sqlite database (default as current)')
-    import_engines.add_argument(mm(POSTGRESQL), dest=ENGINE, action='store_const', const=POSTGRESQL,
-                                help='version is a postgresql database (default as current)')
+    import_ = commands.add_parser(IMPORT, help='import data from a previous version')
+    import_.add_argument(SOURCE, nargs='?',
+                         help='version or uri to import from '
+                              '(version assumes same URI structure as current database); '
+                              'omit to use latest version found locally')
+    import_.add_argument(m(L), mm(LIST), action='store_true', help='list version found locally (and exit)')
     import_.add_argument(mm(DISABLE), action='store_true', help='disable following options (they enable by default)')
     import_.add_argument(mm(DIARY), action='store_true', help='enable (or disable) import of diary data')
     import_.add_argument(mm(ACTIVITIES), action='store_true', help='enable (or disable) import of activity data')
     import_.add_argument(mm(KIT), action='store_true', help='enable (or disable) import of kit data')
     import_.add_argument(mm(CONSTANTS), action='store_true', help='enable (or disable) import of constant data')
     import_.add_argument(mm(SEGMENTS), action='store_true', help='enable (or disable) import of segment data')
+    import_.add_argument(mm(SECTORS), action='store_true', help='enable (or disable) import of sector data')
 
-    garmin = subparsers.add_parser(GARMIN, help='download monitor data from garmin connect')
-    garmin.add_argument(DIR, metavar='DIR', nargs='?', help='the directory where FIT files are stored')
-    garmin.add_argument(mm(USER), metavar='USER', help='garmin connect username')
-    garmin.add_argument(mm(PASS), metavar='PASSWORD', help='garmin connect password')
-    garmin.add_argument(mm(DATE), metavar='DATE', type=to_date, help='date to download')
-    garmin.add_argument(mm(FORCE), action='store_true', help='allow longer date range')
+    delete = commands.add_parser(DELETE, help='delete an activity')
+    delete.add_argument(DATE, help='date of activity to delete')
 
-    calculate = subparsers.add_parser(CALCULATE, help='(re-)calculate statistics')
-    calculate.add_argument(mm(FORCE), action='store_true', help='delete existing statistics')
-    calculate.add_argument(mm(LIKE), action='append', default=[], metavar='PATTERN',
-                           help='run only matching pipeline classes')
-    calculate.add_argument(mm(UNLIKE), action='append', default=[], metavar='PATTERN',
-                           help='exclude matching pipeline classes')
-    calculate.add_argument(START, metavar='START', nargs='?', help='optional start date')
-    calculate.add_argument(FINISH, metavar='FINISH', nargs='?', help='optional finish date (if start also given)')
-    calculate.add_argument(mm(KARG), m(K.upper()), action='append', default=[], metavar='NAME=VALUE',
-                           help='keyword argument to be passed to the pipelines (can be repeated)')
-    calculate.add_argument(mm(WORKER), metavar='ID', type=int,
-                           help='internal use only (identifies sub-process workers)')
-
-    fit = subparsers.add_parser(FIT, help='display contents of fit file')
+    fit = commands.add_parser(FIT, help='display contents of fit file')
     fit_cmds = fit.add_subparsers(title='sub-commands', dest=SUB_COMMAND, required=True)
     fit_grep = fit_cmds.add_parser(GREP, help='show matching entries')
     fit_records = fit_cmds.add_parser(RECORDS, help='show high-level structure (ordered by time)')
@@ -553,7 +502,7 @@ def make_parser(with_noop=False):
         cmd.add_argument(mm(MATCH), type=int, default=-1, help='max number of matches (default -1 for all)')
         cmd.add_argument(mm(COMPACT), action='store_true', help='no space between records')
         cmd.add_argument(mm(CONTEXT), action='store_true', help='display entire record')
-        cmd.add_argument(m(P), mm(PATTERN), nargs='+', metavar='MSG:FLD[=VAL]', required=True,
+        cmd.add_argument(mm(PATTERN), nargs='+', metavar='MSG:FLD[=VAL]', required=True,
                          help='pattern to match (separate from PATH with --)')
 
     def add_fit_high_level(cmd):
@@ -580,7 +529,7 @@ def make_parser(with_noop=False):
         cmd.add_argument(mm(WIDTH), type=int,
                          help='display width')
 
-    fix_fit = subparsers.add_parser(FIX_FIT, help='fix a corrupted fit file')
+    fix_fit = commands.add_parser(FIX_FIT, help='fix a corrupted fit file')
     fix_fit.add_argument(PATH, metavar='PATH', nargs='+', help='path to fit file')
     fix_fit.add_argument(m(W), mm(WARN), action='store_true', help='additional warning messages')
     fix_fit_output = fix_fit.add_argument_group(title='output (default hex to stdout)').add_mutually_exclusive_group()
@@ -624,51 +573,53 @@ def make_parser(with_noop=False):
     fix_fit_params.add_argument(mm(MAX_DELTA_T), type=float, metavar='S',
                                 help='max number of seconds between timestamps')
 
-    thumbnail = subparsers.add_parser(THUMBNAIL,
-                                      help='generate a thumbnail map of an activity')
-    thumbnail.add_argument(ACTIVITY, metavar='ACTIVITY',
-                           help='an activity ID or date')
+    thumbnail = commands.add_parser(THUMBNAIL, help='generate a thumbnail map of an activity')
+    thumbnail.add_argument(ACTIVITY, type=int, metavar='ID', help='an activity ID')
+    add_image_dir(thumbnail)
+    thumbnail.add_argument(mm(DISPLAY), action='store_true', help='display image')
+    thumbnail.add_argument(mm(SECTOR), type=int, nargs='?', metavar='ID', help='mark sector')
+
+    sparkline = commands.add_parser(SPARKLINE, help='generate a sparkline plot for a statistics')
+    sparkline.add_argument(STATISTIC, type=int, metavar='ID', help='the statistics ID')
+    add_image_dir(sparkline)
+    sparkline.add_argument(mm(DISPLAY), action='store_true', help='display image')
+    sparkline.add_argument(mm(INVERT), action='store_true', help='invert image')
+    sparkline.add_argument(mm(SECTOR), type=int, metavar='ID', help='restrict to single sector')
+    sparkline.add_argument(mm(ACTIVITY), type=int, metavar='ID', help='mark activity')
 
     if with_noop:
-        noop = subparsers.add_parser(NO_OP,
-                                     help='used within jupyter (no-op from cmd line)')
+        noop = commands.add_parser(NO_OP, help='used within jupyter (no-op from cmd line)')
 
-    package_fit_profile = subparsers.add_parser(PACKAGE_FIT_PROFILE,
+    package_fit_profile = commands.add_parser(PACKAGE_FIT_PROFILE,
                                                 help='parse and save the global fit profile (dev only)')
     package_fit_profile.add_argument(PATH, metavar='PROFILE',
                                      help='the path to the profile (Profile.xlsx)')
     package_fit_profile.add_argument(m(W), mm(WARN), action='store_true',
                                      help='additional warning messages')
 
-    show_schedule = subparsers.add_parser(SHOW_SCHEDULE, help='print schedule locations in a calendar')
+    show_schedule = commands.add_parser(SHOW_SCHEDULE, help='print schedule locations in a calendar')
     show_schedule.add_argument(SCHEDULE, metavar='SCHEDULE', help='schedule to test')
     show_schedule.add_argument(mm(START), metavar='DATE', help='date to start displaying data')
     show_schedule.add_argument(mm(MONTHS), metavar='N', type=int, help='number of months to display')
 
-    unlock = subparsers.add_parser(UNLOCK, help='remove database locking')
-
     return parser
 
 
-def bootstrap_dir(base, *args, configurator=None, post_config=None):
+def bootstrap_db(user, *args, configurator=None, post_config=None):
+
+    from ..sql.config import Config
     # used in tests, given a base directory
 
-    from ..sql.database import sqlite_uri
-    from ..sql.system import SystemConstant
-
-    args = [mm(BASE), base] + list(args)
+    args = [mm(USER), user] + list(args)
     parser = make_parser()
-    ns = NamespaceWithVariables(parser.parse_args(args=args))
+    ns = NamespaceWithVariables._from_ns(parser.parse_args(args=args), PROGNAME, DB_VERSION)
     if configurator:
-        data = set_global_state(ns)
-        data.sys.set_constant(SystemConstant.DB_URI, sqlite_uri(data.base), force=True)
-        with data.db.session_context() as s:
-            configurator(s, data.base)
+        data = Config(ns)
+        configurator(data)
     args += post_config if post_config else []
-    ns = NamespaceWithVariables(parser.parse_args(args=args))
-    data = set_global_state(ns)
-    data.sys.set_constant(SystemConstant.DB_URI, sqlite_uri(data.base), force=True)
-    return ns, data
+    ns = NamespaceWithVariables._from_ns(parser.parse_args(args=args), PROGNAME, DB_VERSION)
+    data = Config(ns)
+    return data
 
 
 def parse_pairs(pairs, convert=True, multi=False, comma=False):
