@@ -1,9 +1,10 @@
 
 from logging import getLogger
 
-from .help import Markdown
-from ..commands.args import DATE, NAME, VALUE, FORCE, mm, COMMAND, CONSTANTS, SET, SUB_COMMAND, ADD, \
+from ..commands.args import DATE, NAME, VALUE, FORCE, COMMAND, CONSTANTS, SET, SUB_COMMAND, ADD, \
     SHOW, REMOVE, DESCRIPTION, SINGLE, VALIDATE, UNSET, LIST
+from ..common.args import mm
+from ..common.md import Markdown
 from ..sql import ActivityGroup
 from ..sql.tables.constant import Constant, ValidateNamedTuple
 from ..sql.tables.statistic import StatisticJournal, StatisticName, StatisticJournalType
@@ -13,7 +14,7 @@ from ..sql.utils import add
 log = getLogger(__name__)
 
 
-def constants(args, data):
+def constants(config):
     '''
 ## constants
 
@@ -52,9 +53,10 @@ the name of an activity group.
 Names can be matched by SQL patterns.  So FTHR.% matches both FTHR.Run and FTHR.Bike, for example.
 In such a case "entry" in the descriptions above may refer to multiple entries.
     '''
+    args = config.args
     cmd = args[SUB_COMMAND]
     name = None if cmd == LIST else args[NAME]
-    with data.db.session_context() as s:
+    with config.db.session_context() as s:
         if cmd == ADD:
             add_constant(s, name, description=args[DESCRIPTION], single=args[SINGLE], validate=args[VALIDATE])
         else:
@@ -121,7 +123,7 @@ def set_constants(s, constants, date, value, force):
         for constant in constants:
             journals += journal_for_constant(s, constant)
         if journals:
-            log.info(f'Need to delete {len(journals)} ConstantJournal entries')
+            log.info(f'Need to delete {len(journals)} constant values')
             if not force:
                 raise Exception(f'Use {mm(FORCE)} to confirm deletion of prior values')
             for journal in journals:
