@@ -2,17 +2,18 @@
 from logging import getLogger
 
 from .args import PATH, SUB_COMMAND, AFTER_BYTES, LIMIT_BYTES, AFTER_RECORDS, LIMIT_RECORDS, NAME, WIDTH, GREP, \
-    RECORDS, ALL_FIELDS, INTERNAL, ALL_MESSAGES, MESSAGE, FIELD, VALIDATE, MAX_DELTA_T, no, WARN, TABLES, PATTERN, \
+    RECORDS, ALL_FIELDS, INTERNAL, ALL_MESSAGES, MESSAGE, FIELD, VALIDATE, MAX_DELTA_T, WARN, TABLES, PATTERN, \
     COMPACT, CONTEXT, NOT, MATCH, CSV, TOKENS, FIELDS
+from ..common.args import no
 from ..fit.profile.profile import read_fit
 from ..fit.summary import summarize_records, summarize_tables, summarize_grep, summarize_csv, summarize_tokens, \
     summarize_fields
-from ..lib.io import terminal_width
+from ..common.io import terminal_width
 
 log = getLogger(__name__)
 
 
-def fit(args, data):
+def fit(config):
     '''
 ## fit
 
@@ -46,6 +47,7 @@ You may need a `--` between patterns and file paths so that the argument parser
 finish and paths start.
     '''
 
+    args = config.args
     format = args[SUB_COMMAND]
     after_bytes = args[AFTER_BYTES]
     limit_bytes = args[LIMIT_BYTES]
