@@ -4,14 +4,15 @@ from sys import stdout, stderr
 
 from .args import PATH, DROP, OUTPUT, SLICES, RAW, WARN, MIN_SYNC_CNT, MAX_RECORD_LEN, MAX_DROP_CNT, MAX_BACK_CNT, \
     MAX_FWD_LEN, DISCARD, FORCE, VALIDATE, ADD_HEADER, HEADER_SIZE, PROTOCOL_VERSION, PROFILE_VERSION, MAX_DELTA_T, \
-    NAME, FIX_HEADER, FIX_CHECKSUM, NAME_BAD, NAME_GOOD, mm, no, START
+    NAME, FIX_HEADER, FIX_CHECKSUM, NAME_BAD, NAME_GOOD, START
+from ..common.args import mm, no
 from ..fit.fix import fix
 from ..fit.profile.profile import read_fit
 
 log = getLogger(__name__)
 
 
-def fix_fit(args, data):
+def fix_fit(config):
     '''
 ## fix-fit
 
@@ -45,6 +46,7 @@ Will attempt to fix the given file (in the test data from git).
 Will prepend a new 14 byte header, drop the old 14 byte header, and fix the header and checksum values.
     '''
 
+    args = config.args
     check = args[NAME] is not None
     if check:
         name = NAME_GOOD if args[NAME] else NAME_BAD
