@@ -3,12 +3,12 @@ from logging import getLogger
 
 from .args import PATH, WARN
 from ..fit.profile.profile import pickle_profile
-from ..lib.utils import clean_path
+from ..common.io import clean_path
 
 log = getLogger(__name__)
 
 
-def package_fit_profile(args, data):
+def package_fit_profile(config):
     '''
 ## package-fit-profile
 
@@ -19,5 +19,6 @@ to a pickle file that is distributed with this package.
 
 This command is intended for internal use only.
     '''
+    args = config.args
     in_path, warn = clean_path(args[PATH]), args[WARN]
     pickle_profile(in_path, warn=warn)
