@@ -2,14 +2,15 @@
 import webbrowser
 from logging import getLogger
 
-from .args import SUB_COMMAND, SERVICE, STOP, START, STATUS, URI
+from .args import SUB_COMMAND, SERVICE, STOP, START, STATUS
+from ..common.names import URI
 from ..web.server import WebController
 
 
 log = getLogger(__name__)
 
 
-def web(args, data):
+def web(config):
     '''
 ## web
 
@@ -25,12 +26,12 @@ Indicate whether the server is running or not.
 
 Stop the server.
     '''
-    cmd = args[SUB_COMMAND]
-    controller = WebController(args, data)
+    cmd = config.args[SUB_COMMAND]
+    controller = WebController(config)
     if cmd == SERVICE:
         controller.service()  # uri only used for initial config
     elif cmd == STATUS:
-        controller.status()
+        controller.status()  # TODO - broken
     elif cmd == START:
         controller.start(restart=True)
         webbrowser.open(controller.connection_url(), autoraise=False)
