@@ -1,11 +1,14 @@
 from json import dumps
 from logging import getLogger
 
+from sqlalchemy import desc
+
 from ..names import simple_name
+from ..common.names import TIME_ZERO
 from ..sql import ActivityGroup, Constant, Pipeline, PipelineType, StatisticName, StatisticJournalType, \
-    DiaryTopic, DiaryTopicField, Dummy, ActivityTopic, ActivityTopicField
+    DiaryTopic, DiaryTopicField, ActivityTopic, ActivityTopicField
 from ..sql.tables.constant import ValidateNamedTuple
-from ..sql.types import long_cls
+from ..sql.types import long_cls, short_cls
 
 log = getLogger(__name__)
 
@@ -51,33 +54,42 @@ def add(s, instance):
     return instance
 
 
-def add_activity_group(s, name, sort, description=None):
+def add_activity_group(s, title, sort, description=None):
     '''
     Add an activity type to the configuration.
 
     These are used to group activities (and related statistics).
     So typical entries might be for cycling, running, etc.
     '''
-    return add(s, ActivityGroup(name=name, sort=sort, description=description))
+    return add(s, ActivityGroup(name=title, sort=sort, description=description))
 
 
-def add_statistics(s, cls, sort, **kargs):
-    '''
-    Add a class to the statistics pipeline.
-
-    The pipeline classes are invoked when the diary is modified and when activities are added.
-    They detect new data and calculate appropriate statistics.
-
-    The sort argument fixes the order in which the classes are instantiated and called and can
-    be an integer or a callable (that returns an integer) like Counter above.
-
-    The kargs are passed to the constructor and so can be used to customize the processing.
-    '''
-    log.debug(f'Adding statistic pipeline {cls}')
-    return add(s, Pipeline(cls=cls, type=PipelineType.CALCULATE, sort=sort, kargs=kargs))
+def add_pipeline(s, cls, type, blocked_by=None, **kargs):
+    pipeline = add(s, Pipeline(cls=cls, type=type, kargs=kargs))
+    for blocker in blocked_by or ():
+        if isinstance(blocker, Pipeline):
+            log.debug(f'{short_cls(blocker.cls)} blocks {short_cls(cls)}')
+            pipeline.blocked_by.append(blocker)
+        else:
+            q = s.query(Pipeline). \
+                filter(Pipeline.type == pipeline.type,
+                       Pipeline.cls == blocker)
+            if q.count():
+                for instance in q.all():
+                    pipeline.blocked_by.append(instance)
+                log.debug(f'{blocker} blocks {cls}')
+            else:
+                raise Exception(f'{short_cls(blocker)} is not a current pipeline')
+    return pipeline
 
 
-def add_displayer(s, cls, sort, **kargs):
+def add_next_pipeline(s, cls, type, **kargs):
+    previous = s.query(Pipeline).filter(Pipeline.type == type).order_by(desc(Pipeline.id)).first()
+    if previous: previous = [previous]
+    return add_pipeline(s, cls, type, blocked_by=previous, **kargs)
+
+
+def add_displayer(s, cls, **kargs):
     '''
     Add a class to the diary pipeline.
 
@@ -89,11 +101,11 @@ def add_displayer(s, cls, sort, **kargs):
 
     The kargs are passed to the constructor and so can be used to customize the processing.
     '''
-    log.debug(f'Adding displayer pipeline {cls}')
-    return add(s, Pipeline(cls=cls, type=PipelineType.DISPLAY, sort=sort, kargs=kargs))
+    log.debug(f'Adding displayer pipeline {short_cls(cls)}')
+    return add_next_pipeline(s, cls, PipelineType.DISPLAY, **kargs)
 
 
-def add_activity_displayer_delegate(s, cls, sort, **kargs):
+def add_activity_displayer_delegate(s, cls, **kargs):
     '''
     Add a class to the activity displayer pipeline.
 
@@ -104,45 +116,29 @@ def add_activity_displayer_delegate(s, cls, sort, **kargs):
 
     The kargs are passed to the constructor and so can be used to customize the processing.
     '''
-    log.debug(f'Adding activity displayer pipeline {cls}')
-    return add(s, Pipeline(cls=cls, type=PipelineType.DISPLAY_ACTIVITY, sort=sort, kargs=kargs))
+    log.debug(f'Adding activity displayer pipeline {short_cls(cls)}')
+    return add_next_pipeline(s, cls, PipelineType.DISPLAY_ACTIVITY, **kargs)
 
 
-def add_monitor(s, cls, sort, **kargs):
+def add_process(s, cls, blocked_by=None, **kargs):
     '''
-    Add a class to the monitor pipeline.
+    Add a class to the process pipeline.
 
     The pipeline classes are invoked when activities are imported from FIT files.
-    They read the files and create MonitorJournal entries and associated statistics.
+    They read the files and create journal entries and associated statistics.
 
-    The sort argument fixes the order in which the classes are instantiated and called and can
+    The blocked_by argument fixes the order in which the classes are instantiated and called and can
     be an integer or a callable (that returns an integer) like Counter above.
 
     The kargs are passed to the constructor and so can be used to customize the processing.
     '''
-    log.debug(f'Adding monitor pipeline {cls}')
-    return add(s, Pipeline(cls=cls, type=PipelineType.READ_MONITOR, sort=sort, kargs=kargs))
-
-
-def add_activities(s, cls, sort, **kargs):
-    '''
-    Add a class to the activities pipeline.
-
-    The pipeline classes are invoked when activities are imported from FIT files.
-    They read the files and create ActivityJournal entries and associated statistics.
-
-    The sort argument fixes the order in which the classes are instantiated and called and can
-    be an integer or a callable (that returns an integer) like Counter above.
-
-    The kargs are passed to the constructor and so can be used to customize the processing.
-    '''
-    log.debug(f'Loading activity pipeline {cls}')
-    return add(s, Pipeline(cls=cls, type=PipelineType.READ_ACTIVITY, sort=sort, kargs=kargs))
+    log.debug(f'Adding process pipeline {short_cls(cls)}')
+    return add_pipeline(s, cls, PipelineType.PROCESS, blocked_by=blocked_by, **kargs)
 
 
 def add_constant(s, title, value, description=None, units=None, name=None,
                  statistic_journal_type=StatisticJournalType.INTEGER, activity_group=None,
-                 time=0.0, single=False, validate_cls=None, validate_args=None, validate_kargs=None):
+                 time=TIME_ZERO, single=False, validate_cls=None, validate_args=None, validate_kargs=None):
     '''
     Add a constant.
 
@@ -280,12 +276,3 @@ def add_activity_topic_field(s, activity_topic, title, sort, type, activity_grou
     return add(s, ActivityTopicField(activity_topic=activity_topic, sort=sort, model=model,
                                      statistic_name=statistic_name))
 
-
-def add_loader_support(s):
-    '''
-    Add 'dummy' value used by loader.
-    '''
-    log.debug('Adding dummy source')
-    dummy_source = add(s, Dummy())
-    dummy_name = add(s, StatisticName(name=Dummy.DUMMY, owner=dummy_source,
-                                      statistic_journal_type=StatisticJournalType.STATISTIC))
