@@ -1,16 +1,17 @@
 
+from .climb import climb_sources
 from .constraint import constrained_sources
 from .frame import session, nearby_activities, bookmarks, present, linear_resample_time, \
     groups_by_time, transform, drop_empty, read_query
 from .heart_rate import *
-from .lib import chisq, fit, inplace_decay
+from .lib import inplace_decay
 from .plot import col_to_boxstats, box_plot, line_plotter, dot_plotter, bar_plotter, add_climbs, multi_plot, \
     multi_dot_plot, multi_bar_plot, multi_line_plot, map_thumbnail, map_intensity, map_plot, histogram_plot, \
     cumulative_plot, htile, vtile, comparison_line_plot, map_intensity_signed, add_hr_zones, \
     add_multi_line_at_index, std_distance_time_plot, add_band, get_renderer, add_curve, add_climb_zones
-from .power import fit_power
 from .query import Statistics, std_health_statistics, std_activity_statistics
 from ..pipeline.display.activity.nearby import nearby_earlier, nearby_any_time
+from .fit import fit
 
 # avoid cleaning of imports
 nearby_earlier, nearby_any_time, session, Statistics
