@@ -1,15 +1,16 @@
-
-import datetime as dt
 from collections import namedtuple
+from functools import cmp_to_key
 from itertools import groupby
 from logging import getLogger
 
-import numpy as np
+from sqlalchemy import or_
 
 from .frame import linear_resample, present
-from ..lib.data import nearest_index, get_index_loc, safe_yield, safe_none
+from ..common.math import is_nan
+from ..lib.data import nearest_index, get_index_loc, safe_yield
 from ..names import Names as N
-from ..sql import StatisticName, StatisticJournal, Source
+from ..sql import StatisticName, StatisticJournal, Source, ActivityJournal
+from ..sql.tables.sector import SectorJournal
 
 log = getLogger(__name__)
 
@@ -28,7 +29,7 @@ CLIMB_CATEGORIES = {MIN_CLIMB_ELEVATION: '4', 160: '3', 320: '2', 640: '1', 800:
 PERCENT = 100 / 1000
 
 # trade-off between pure elevation (0) and pure gradient (1)
-CLIMB_PHI = 0.6
+CLIMB_PHI = 0.7
 
 Climb = namedtuple('Climb', 'phi, min_elevation, min_gradient, max_gradient, max_reversal',
                    defaults=(CLIMB_PHI, MIN_CLIMB_ELEVATION, MIN_CLIMB_GRADIENT,
@@ -121,7 +122,7 @@ def biggest_climb(df, params=Climb(), grid=10):
     # returns (score, dlo, dhi))
     # use distances (indices) rather than ilocs because we're subdividing the data
     if len(df) > 100 * grid:
-        score, lo, hi = search(df.iloc[::grid].copy())
+        score, lo, hi = search(df.iloc[::grid].copy(), grid=True)
         if score:
             # need to pass through iloc to extend range
             ilo, ihi = get_index_loc(df, lo), get_index_loc(df, hi)
@@ -132,9 +133,9 @@ def biggest_climb(df, params=Climb(), grid=10):
         return search(df, params=params)
 
 
-def search(df, params=Climb()):
+def search(df, params=Climb(), grid=False):
     # returns (score, dlo, dhi)
-    # use times (indices) rather than ilocs because we're subdividing the data
+    # use distance (indices) rather than ilocs because we're subdividing the data
     max_score, max_indices, d = 0, (None, None), df.index[1] - df.index[0]
     for offset in range(len(df)-1, 0, -1):
         df[N._delta(N.ELEVATION)] = df[N.ELEVATION].diff(offset)
@@ -142,48 +143,64 @@ def search(df, params=Climb()):
         min_elevation = max(params.min_elevation, params.min_gradient * d_distance / PERCENT)
         if df[N._delta(N.ELEVATION)].max() > min_elevation:  # avoid some work
             # factor of 1000 below to convert km to m
-            df[SCORE] = (df[N._delta(N.ELEVATION)] / (1000 * d_distance)) ** params.phi
+            df[SCORE] = df[N._delta(N.ELEVATION)] / ((1000 * d_distance) ** params.phi)
             score = df.loc[df[N._delta(N.ELEVATION)] > min_elevation, SCORE].max()
-            if not np.isnan(score) and score > max_score:
+            if not is_nan(score) and score > max_score:
                 max_score = score
                 hi = df.loc[df[SCORE] == max_score].index[0]  # arbitrarily pick one if tied (error here w item())
                 lo = df.index[get_index_loc(df, hi) - offset]
+                if not grid:
+                    # step inwards one location from each end
+                    # (so that we have some 'extra' to aid with intersections)
+                    lo = df.index[get_index_loc(df, hi) - (offset-1)]
+                    hi = df.index[get_index_loc(df, hi) - 1]
                 max_indices = (lo, hi)
     return max_score, max_indices[0], max_indices[1]
 
 
 def climbs_for_activity(s, ajournal):
 
-    from ..pipeline.owners import ActivityCalculator
+    from ..pipeline.calculate.sector import SectorCalculator
+    from ..pipeline.calculate import ActivityCalculator
 
-    total = s.query(StatisticJournal).join(StatisticName, Source). \
+    total = s.query(StatisticJournal). \
+        join(StatisticName).join(Source). \
         filter(StatisticName.name == N.TOTAL_CLIMB,
                StatisticJournal.time == ajournal.start,
                StatisticName.owner == ActivityCalculator,
                Source.activity_group == ajournal.activity_group).order_by(StatisticJournal.time).one_or_none()
-    statistics = s.query(StatisticJournal).join(StatisticName, Source). \
-        filter(StatisticName.name.like(N.CLIMB_ANY),
+    query = s.query(StatisticJournal). \
+        join(StatisticName).join(Source). \
+        join(SectorJournal, SectorJournal.id == Source.id). \
+        filter(or_(StatisticName.name.like(N.CLIMB_ANY),
+                   StatisticName.name == N.VERTICAL_POWER),
                StatisticJournal.time >= ajournal.start,
                StatisticJournal.time <= ajournal.finish,
-               StatisticName.owner == ActivityCalculator,
-               Source.activity_group == ajournal.activity_group).order_by(StatisticJournal.time).all()
-    return total, sorted((dict((statistic.statistic_name.name, statistic) for statistic in climb_statistics)
-                          for _, climb_statistics in groupby(statistics, key=lambda statistic: statistic.time)),
-                         key=lambda climb: climb[N.CLIMB_ELEVATION].value, reverse=True)
+               StatisticName.owner == SectorCalculator,
+               Source.activity_group == ajournal.activity_group).order_by(StatisticJournal.time)
+    sjournals = query.all()
 
+    def make_climb(sjournals):
+        sjournals = list(sjournals)
+        climb = {sjournal.statistic_name.name: sjournal for sjournal in sjournals}
+        climb['start-distance'] = sjournals[0].source.start_distance
+        climb['sector-journal-id'] = sjournals[0].source.id
+        climb['sector-id'] = sjournals[0].source.sector_id
+        return climb
 
-@safe_none
-def add_climb_stats(df, climbs):
-    for climb in climbs:
-        finish = climb[N.TIME]
-        start = finish - dt.timedelta(seconds=climb[N.CLIMB_TIME])
-        if N.POWER_ESTIMATE in df.columns:
-            # mean() returns a series!
-            power = df.loc[start:finish, [N.POWER_ESTIMATE]].mean()[0]
-            if not np.isnan(power):
-                climb[N.CLIMB_POWER] = power
-            else:
-                log.warning(f'Invalid {N.POWER_ESTIMATE} in climb data')
+    def cmp_climb(a, b):
+        # if start distances are similar, order by elevation
+        if abs(a['start-distance'] - b['start-distance']) / (a['start-distance'] + b['start-distance']) > 0.03:
+            return a['start-distance'] - b['start-distance']
         else:
-            log.warning(f'Missing {N.POWER_ESTIMATE} in climb data')
+            return a[N.CLIMB_ELEVATION].value - b[N.CLIMB_ELEVATION].value
 
+    return total, sorted((make_climb(grouped)
+                          for _, grouped in groupby(sjournals, key=lambda sjournal: sjournal.time)),
+                         key=cmp_to_key(cmp_climb))
+
+
+def climb_sources(s, activity_journal, activity_group=None):
+    if not isinstance(activity_journal, Source):
+        activity_journal = ActivityJournal.at(s, activity_journal, activity_group=activity_group)
+    return s.query(SectorJournal).filter(SectorJournal.activity_journal_id == activity_journal.id).all()
