@@ -1,7 +1,7 @@
 
 from logging import getLogger
 
-from ..lib.date import to_time
+from ..common.date import to_time
 from ..sql import ActivityBookmark, ActivityGroup, StatisticName
 from ..sql.database import connect
 
@@ -32,13 +32,13 @@ class CoastingBookmark:
 
     def __find(self, s, kit, min_time, max_cadence, min_speed, group):
 
-        from ..pipeline.owners import ActivityReader, SegmentReader
+        from ..pipeline.owners import ActivityReader
 
         group = ActivityGroup.from_name(s, group)
-        cadence_statistic = StatisticName.from_name(s, 'Cadence', SegmentReader, group)
-        distance_statistic = StatisticName.from_name(s, 'Distance', SegmentReader, group)
-        speed_statistic = StatisticName.from_name(s, 'Speed', SegmentReader, group)
-        kit_statistic = StatisticName.from_name(s, ActivityReader.Kit, SegmentReader, group)
+        cadence_statistic = StatisticName.from_name(s, 'Cadence', ActivityReader, group)
+        distance_statistic = StatisticName.from_name(s, 'Distance', ActivityReader, group)
+        speed_statistic = StatisticName.from_name(s, 'Speed', ActivityReader, group)
+        kit_statistic = StatisticName.from_name(s, ActivityReader.Kit, ActivityReader, group)
         s.execute('PRAGMA automatic_index=OFF;')
         for row in s.execute('''
 select t.activity_journal_id, s.time, f.time
@@ -130,6 +130,6 @@ if __name__ == '__main__':
     
     These are then used in the fit_power_parameters template to estimate CdA and Crr. 
     '''
-    ns, db = connect(['-v 4'])
+    db = connect(['-v 4'])
 #    CoastingBookmark(db).run('cotic', 60, 20, 0, 'bike', constraint='cotic 60/20/0')
     CoastingBookmark(db).run('cotic', 15, 5, 10, 'bike', constraint='cotic 15/5/10')
