@@ -29,7 +29,7 @@ def session(*args):
     '''
     Create a database session (used in Jupyter templates)
     '''
-    ns, db = connect(args)
+    db = connect(args)
     return db.session()
 
 
@@ -151,7 +151,3 @@ def drop_empty(df):
         if df[column].dropna().empty:
             df = df.drop(columns=[column])
     return df
-
-
-def valid(value):
-    return value == value and value is not None
