@@ -1,20 +1,21 @@
-
+import datetime as dt
 from logging import getLogger
 
 import numpy as np
 import pandas as pd
+import pytz
 from sqlalchemy import asc, desc, distinct
 from sqlalchemy.orm import aliased
 
-from ch2.data import read_query
+from ..common.date import YMD
+from ..common.log import log_current_exception
+from ..common.names import TIME_ZERO
+from ..data import read_query
 from ..data import session, present
-from ..lib import local_date_to_time, to_date, time_to_local_time
-from ..lib.date import YMD, format_seconds
-from ..lib.log import log_current_exception
+from ..lib import time_to_local_time
 from ..lib.utils import timing
-from ..names import Names as N, like, MED_WINDOW, SPACE
-from ..sql import StatisticName, ActivityGroup, StatisticJournal, ActivityTimespan, ActivityJournal, Source, \
-    ActivityTopic
+from ..names import Names as N, like, MED_WINDOW, SPACE, simple_name
+from ..sql import StatisticName, ActivityGroup, StatisticJournal, ActivityTimespan, ActivityJournal, Source
 from ..sql.tables.statistic import STATISTIC_JOURNAL_CLASSES
 from ..sql.types import short_cls
 
@@ -23,7 +24,7 @@ log = getLogger(__name__)
 
 class Statistics:
 
-    def __init__(self, s, start=None, finish=None, sources=None, with_timespan=False, with_source=False,
+    def __init__(self, s, start=None, finish=None, sources=None, with_timespan=False, with_sources=False,
                  activity_journal=None, activity_group=None, bookmarks=None, warn_over=1):
         '''
         Specify any general constraints when constructing the object, then request particular statistics
@@ -43,7 +44,7 @@ class Statistics:
                 activity_journal = ActivityJournal.at(s, activity_journal, activity_group=activity_group)
             self.__sources.append(activity_journal)
         self.__with_timespan = with_timespan
-        self.__with_source = with_source
+        self.__with_sources = with_sources
         self.__activity_group = activity_group
         self.__warn_over = warn_over
         self.__statistic_names = {}
@@ -78,7 +79,7 @@ class Statistics:
 
     def __columns(self, type_class, label):
         columns = [type_class.time.label(N.INDEX), type_class.value.label(label)]
-        if self.__with_source:
+        if self.__with_sources:
             columns += [type_class.source_id.label(N._src(label))]
         return columns
 
@@ -129,7 +130,7 @@ class Statistics:
         if self.__sources:
             q = q.filter(StatisticJournal.source_id.in_([source.id for source in self.__sources]))
         elif self.__activity_group:
-            # only use of sources not specified separately (since those fix groups anyway)
+            # only use if sources not specified separately (since those fix groups anyway)
             source = aliased(Source)
             q = q.join(source, source.id == StatisticJournal.source_id). \
                 filter(source.activity_group_id == self.__activity_group.id)
@@ -189,6 +190,7 @@ class Data:
             self.df[name] = self.df[name] * scale
         if median:
             self.df[name] = self.df[name].rolling(median, min_periods=1).median()
+        return self
 
     def __rename(self, name, new_name, scale=1.0, median=None):
         log.debug(f'{name} -> {new_name}')
@@ -250,7 +252,7 @@ class Data:
         set_times_from_index(self.df)
         return self
 
-    def __coallesce(self, columns, name, delete=False):
+    def __coalesce(self, columns, name, delete=False):
         log.debug(f'Coallescing {columns} for {name}')
         df = self.df[columns].copy()
         df.fillna(method='ffill', axis='columns', inplace=True)
@@ -259,7 +261,7 @@ class Data:
         if delete:
             self.df.drop(columns=columns, inplace=True)
 
-    def coallesce_groups(self, *names, delete=False):
+    def coalesce_groups(self, *names, delete=False):
         '''
         Merge columns named with groups.
         '''
@@ -270,10 +272,10 @@ class Data:
             if name in self.df.columns:
                 raise Exception(f'{name} already exists')
             columns = like(name + ':%', self.df.columns)
-            self.__coallesce(columns, name, delete=delete)
+            self.__coalesce(columns, name, delete=delete)
         return self
 
-    def coallesce_like(self, map, delete=False):
+    def coalesce_like(self, map, delete=False):
         '''
         Merge columns that match.  map is {pattern: result}
         '''
@@ -283,7 +285,46 @@ class Data:
             if name in self.df.columns:
                 raise Exception(f'{name} already exists')
             columns = like(pattern, self.df.columns)
-            self.__coallesce(columns, name, delete=delete)
+            self.__coalesce(columns, name, delete=delete)
+        return self
+
+    def __any_source(self):
+        prefix = simple_name(N._src(''))
+        for name in self.df.columns:
+            if name.startswith(prefix):
+                return name
+        raise Exception('No sources (use with_sources=True)')
+
+    def __concat_sources(self, name, gap, drop_index):
+        src = self.__any_source()
+        self.df.dropna(subset=[src], inplace=True)
+        self.df.sort_index(inplace=True)
+        self.df['index'] = self.df.index
+        irows = list(self.df.groupby(src).agg({'index': ['min', 'max']}).iterrows())
+        ilo, ihi = zip(*[row[1] for row in reversed(irows)])
+        for iend, istart in zip(ihi[1:], ilo):
+            end = self.df[name][self.df['index'] == iend][0]
+            start = self.df[name][self.df['index'] == istart][0]
+            subset = self.df['index'] > iend
+            self.df.loc[subset, name] = self.df.loc[subset, name] - (start - end - gap)
+        if drop_index:
+            self.df.drop(columns=['index'], inplace=True)
+
+    def concat(self, name, gap):
+        self.__concat_sources(name, gap, True)
+        return self
+
+    def concat_index(self, gap):
+        self.__concat_sources('index', dt.timedelta(seconds=gap), False)
+        self.df.set_index('index', inplace=True)
+        return self
+
+    def without_sources(self):
+        prefix = simple_name(N._src(''))
+        for name in self.df.columns:
+            if name.startswith(prefix):
+                log.debug(f'Dropping {name}')
+                self.df.drop(columns=[name], inplace=True)
         return self
 
 
@@ -294,14 +335,17 @@ def set_times_from_index(df):
 
 def std_health_statistics(s, freq='1h'):
 
-    from ..pipeline.owners import RestHRCalculator, ResponseCalculator, ActivityCalculator, MonitorCalculator
+    from ..pipeline.owners import RestHRCalculator, ResponseCalculator, ActivityCalculator, StepsCalculator
 
+    # 2 days to skip constants etc with time zones
     start = s.query(StatisticJournal.time). \
-        filter(StatisticJournal.time > local_date_to_time(to_date('1970-01-03'))). \
+        filter(StatisticJournal.time > TIME_ZERO + dt.timedelta(days=2)). \
         order_by(asc(StatisticJournal.time)).limit(1).scalar()
     finish = s.query(StatisticJournal.time).order_by(desc(StatisticJournal.time)).limit(1).scalar()
 
-    stats = pd.DataFrame(index=pd.date_range(start=start, end=finish, freq=freq))
+    # convert to UTC because we may have postgres timezones (at UTC, but incompatible)
+    stats = pd.DataFrame(index=pd.date_range(start=start.replace(tzinfo=pytz.UTC),
+                                             end=finish.replace(tzinfo=pytz.UTC), freq=freq))
     set_times_from_index(stats)
 
     stats = Statistics(s). \
@@ -310,13 +354,13 @@ def std_health_statistics(s, freq='1h'):
 
     stats = Statistics(s). \
         by_name(RestHRCalculator, N.REST_HR). \
-        by_name(MonitorCalculator, N.DAILY_STEPS). \
+        by_name(StepsCalculator, N.DAILY_STEPS). \
         by_name(ActivityCalculator, N._delta(N.DEFAULT_ANY), like=True).with_. \
         rename_with_units(N.REST_HR).into(stats, tolerance='30m')
 
     stats = Statistics(s).\
         by_group(ActivityCalculator, N.ACTIVE_TIME, N.ACTIVE_DISTANCE).with_. \
-        coallesce_groups(N.ACTIVE_TIME, N.ACTIVE_DISTANCE). \
+        coalesce_groups(N.ACTIVE_TIME, N.ACTIVE_DISTANCE). \
         rename_with_units(N.ACTIVE_TIME, N.ACTIVE_DISTANCE). \
         copy({N.ACTIVE_TIME_S: N.ACTIVE_TIME_H}, scale=1 / 3600). \
         into(stats, tolerance='30m')
@@ -331,13 +375,13 @@ def std_activity_statistics(s, activity_journal, activity_group=None):
     from ..pipeline.calculate.elevation import ElevationCalculator
     from ..pipeline.calculate.impulse import ImpulseCalculator
     from ..pipeline.calculate.power import PowerCalculator
-    from ..pipeline.read.segment import SegmentReader
+    from ..pipeline.read.activity import ActivityReader
 
     if not isinstance(activity_journal, ActivityJournal):
         activity_journal = ActivityJournal.at(s, activity_journal, activity_group=activity_group)
 
     stats = Statistics(s, activity_journal=activity_journal, with_timespan=True). \
-        by_name(SegmentReader, N.LATITUDE, N.LONGITUDE, N.SPHERICAL_MERCATOR_X, N.SPHERICAL_MERCATOR_Y,
+        by_name(ActivityReader, N.LATITUDE, N.LONGITUDE, N.SPHERICAL_MERCATOR_X, N.SPHERICAL_MERCATOR_Y,
                 N.DISTANCE, N.SPEED, N.CADENCE, N.ALTITUDE, N.HEART_RATE).with_. \
         rename_with_units(N.LATITUDE, N.LONGITUDE, N.DISTANCE, N.SPEED, N.CADENCE, N.ALTITUDE, N.HEART_RATE). \
         copy({N.SPEED_MS: N.MED_SPEED_KMH}, scale=3.6, median=MED_WINDOW). \
@@ -355,38 +399,39 @@ def std_activity_statistics(s, activity_journal, activity_group=None):
         drop_prefix(N.DEFAULT + SPACE).into(stats, tolerance='10s', interpolate=True)
 
     stats = Statistics(s, activity_journal=activity_journal). \
-        by_name(PowerCalculator, N.POWER_ESTIMATE).with_. \
+        by_name(PowerCalculator, N.POWER_ESTIMATE, N.VERTICAL_POWER).with_. \
         rename_with_units(). \
-        copy({N.POWER_ESTIMATE_W: N.MED_POWER_ESTIMATE_W}, median=MED_WINDOW). \
+        copy({N.POWER_ESTIMATE_W: N.MED_POWER_ESTIMATE_W,
+              N.VERTICAL_POWER_W: N.MED_VERTICAL_POWER_W}, median=MED_WINDOW). \
         into(stats, tolerance='1s')
 
     return stats
 
 
-if __name__ == '__main__':
-
-    from ..pipeline.owners import ActivityCalculator, SegmentReader, MonitorCalculator
-
-    s = session('-v5')
-
-    with timing('select'):
-        # df = std_health_statistics(s)
-        # df = std_activity_statistics(s, '2020-05-15', 'road')
-        # df = Statistics(s).like(N.CLIMB_ANY, owner=ActivityCalculator).from_(activity_journal='2020-05-15').by_group().df
-        # df = Statistics(s).for_(N.ACTIVE_DISTANCE, owner=ActivityCalculator).by_group()
-        # acc = Accumulator(s, sources=[ActivityJournal.at(s, '2020-05-15')])
-        # acc = Accumulator(s, with_timespan=True, sources=[ActivityJournal.at(s, '2020-05-15')])
-        # acc = Accumulator(s, with_source=True)
-        # acc.by_name(ActivityCalculator, N.CLIMB_ANY, like=True)
-        # df = acc.df
-        df = Statistics(s). \
-            by_name(ActivityTopic, N.NAME). \
-            by_name(ActivityCalculator, N.ACTIVE_DISTANCE, N.ACTIVE_TIME, N.TOTAL_CLIMB).with_. \
-            copy({N.ACTIVE_DISTANCE: N.ACTIVE_DISTANCE_KM}).add_times().df
-        print(df.describe())
-        df['Duration'] = df[N.ACTIVE_TIME].map(format_seconds)
-
-    print(df)
-    print(df.describe())
-    print(df.columns)
-    print(df[N.REST_HR_BPM].describe())
+def interpolate(s, source, statistic_name, statistic_owner, time, activity_group=None):
+    if not isinstance(source, Source):
+        source = ActivityJournal.at(s, source, activity_group=activity_group)
+    before = s.query(StatisticJournal). \
+        join(StatisticName). \
+        filter(StatisticName.name == statistic_name,
+               StatisticName.owner == statistic_owner,
+               StatisticJournal.time <= time,
+               StatisticJournal.source == source). \
+        order_by(desc(StatisticJournal.time)).first()
+    after = s.query(StatisticJournal). \
+        join(StatisticName). \
+        filter(StatisticName.name == statistic_name,
+               StatisticName.owner == statistic_owner,
+               StatisticJournal.time >= time,
+               StatisticJournal.source == source). \
+        order_by(asc(StatisticJournal.time)).first()
+    if before is None or after is None:
+        return None
+    if before.time == time:
+        return before.value
+    if after.time == time:
+        return after.value
+    dt = (after.time - before.time).total_seconds()
+    ta = (after.time - time).total_seconds()
+    tb = (time - before.time).total_seconds()
+    return (ta * before.value + tb * after.value) / dt
