@@ -9,7 +9,7 @@ from .format.read import filtered_records, filtered_tokens
 from .format.records import no_bad_values, fix_degrees, append_units, no_unknown_fields, join_values, \
     to_hex, no_filter, merge_duplicates
 from ..commands.args import RECORDS, FIELDS, CSV, TABLES, GREP, TOKENS
-from ..lib.io import terminal_width
+from ..common.io import terminal_width
 from ..lib.utils import unique
 
 log = getLogger(__name__)
