@@ -29,11 +29,11 @@ def all_activities(start, finish):
     s = session('-v2')
     maps = [map_thumbnail(100, 120, data)
             for data in (Statistics(s, activity_journal=aj).
-                             by_name(SegmentReader, N.SPHERICAL_MERCATOR_X, N.SPHERICAL_MERCATOR_Y).
+                             by_name(ActivityReader, N.SPHERICAL_MERCATOR_X, N.SPHERICAL_MERCATOR_Y).
                              by_name(ActivityCalculator, N.ACTIVE_DISTANCE, N.ACTIVE_TIME).df
                          for aj in s.query(ActivityJournal).
                              filter(ActivityJournal.start >= local_date_to_time(start),
-                                    ActivityJournal.start < local_date_to_time(finish)).
+                                    ActivityJournal.start <= local_date_to_time(finish)).
                              order_by(ActivityJournal.start.desc()).all())
             if len(data[N.SPHERICAL_MERCATOR_X].dropna()) > 10]
     print(f'Found {len(maps)} activities')
