@@ -1,46 +1,19 @@
 
 import re
-from hashlib import md5
 from logging import getLogger
 from os import stat
-from pathlib import Path
-from shutil import get_terminal_size
 
 from sqlalchemy import desc
 
-from .date import to_time
+from ..common.date import to_time
+from ..common.io import file_hash
+from ..common.names import TIME_ZERO
 from ..sql.tables.file import FileScan, FileHash
 
-
 log = getLogger(__name__)
 
 
-def terminal_width(width=None):
-    width = get_terminal_size()[0] if width is None else width
-    if width is None:
-        log.warning('No terminal width available')
-    else:
-        log.debug(f'Terminal width is {width}')
-    return width
-
-
-# https://stackoverflow.com/a/3431838
-def file_hash(file_path):
-    hash = md5()
-    with open(file_path, "rb") as f:
-        for chunk in iter(lambda: f.read(4096), b''):
-            hash.update(chunk)
-    return hash.hexdigest()
-
-
-def data_hash(data):
-    if isinstance(data, str): data = data.encode('utf-8')
-    hash = md5()
-    hash.update(data)
-    return hash.hexdigest()
-
-
-def modified_file_scans(s, paths, owner, force=False):
+def modified_file_scans(s, paths, owner):
 
     modified = []
 
@@ -49,6 +22,10 @@ def modified_file_scans(s, paths, owner, force=False):
         # log.debug(f'Scanning {path}')
         last_modified = to_time(stat(path).st_mtime)
         hash = file_hash(path)
+        file_scan_from_hash = s.query(FileScan).\
+            join(FileHash).\
+            filter(FileHash.hash == hash,
+                   FileScan.owner == owner).one_or_none()
         file_scan_from_path = s.query(FileScan). \
             filter(FileScan.path == path,
                    FileScan.owner == owner).one_or_none()
@@ -56,15 +33,22 @@ def modified_file_scans(s, paths, owner, force=False):
         # get last scan and make sure it's up-to-date
         if file_scan_from_path:
             if hash != file_scan_from_path.file_hash.hash:
-                log.warning('File at %s appears to have changed since last read on %s')
+                log.warning(f'File at {path} appears to have changed since last read on '
+                            f'{file_scan_from_path.last_scan}')
                 file_scan_from_path.file_hash = FileHash.get_or_add(s, hash)
-                file_scan_from_path.last_scan = 0.0
+                file_scan_from_path.last_scan = TIME_ZERO
+        elif file_scan_from_hash:
+            log.warning(f'File at {path} already exists at {file_scan_from_hash} - skipping')
+            continue
         else:
             file_scan_from_path = FileScan.add(s, path, owner, hash)
             s.flush()  # want this to appear in queries below
 
         # only look at hash if we are going to process anyway
-        if force or last_modified > file_scan_from_path.last_scan:
+        if last_modified > file_scan_from_path.last_scan:
+
+            log.debug(f'File at {path} was modified on {last_modified} '
+                      f'which is after last read on {file_scan_from_path.last_scan}')
 
             file_scan_from_hash = s.query(FileScan).\
                 join(FileHash).\
@@ -78,7 +62,7 @@ def modified_file_scans(s, paths, owner, force=False):
                 # update the path to avoid triggering in future
                 file_scan_from_path.last_scan = file_scan_from_hash.last_scan
 
-            if force or last_modified > file_scan_from_hash.last_scan:
+            if last_modified > file_scan_from_hash.last_scan:
                 modified.append(file_scan_from_hash)
 
     s.commit()
@@ -95,5 +79,3 @@ def split_fit_path(path):
         return path[:-4] + '*' + path[-4:], None
 
 
-def touch(path):
-    Path(path).touch()
