@@ -4,15 +4,16 @@ from contextlib import contextmanager
 from itertools import zip_longest, groupby
 from logging import getLogger
 from os import getpid, close, execl
-from os.path import split, normpath, expanduser
+from os.path import split
 from pprint import PrettyPrinter
 from sys import executable, argv
 from time import sleep
 
 from psutil import Process
 
-from .date import now
-from ..names import Units, UNDEF
+from ..common.date import now
+from ..common.names import UNDEF
+from ..names import U
 
 log = getLogger(__name__)
 
@@ -134,24 +135,24 @@ def short_str(x):
 
 def format_metres(dist):
     if dist < 1000:
-        return str(int(dist)) + Units.M
+        return str(int(dist)) + U.M
     else:
         return format_km(dist / 1000)
 
 
 def format_km(dist):
-    return f'{dist:.1f}{Units.KM}'
+    return f'{dist:.1f}{U.KM}'
 
 
 def format_percent(pc):
     if pc > 0.5:
-        return f'{pc:.1f}{Units.PC}'
+        return f'{pc:.1f}{U.PC}'
     else:
-        return f'{pc:.2f}{Units.PC}'
+        return f'{pc:.2f}{U.PC}'
 
 
 def format_watts(power):
-    return str(int(power)) + Units.W
+    return str(int(power)) + U.W
 
 
 def groupby_tuple(iterable, key=None):
@@ -206,11 +207,6 @@ def restart_self():
     # no need to exit as we do not return
 
 
-def clean_path(path):
-    # don't use realpath here since it messes up inside docker
-    return normpath(expanduser(path))
-
-
 def slow_warning(msg, n=3, pause=1):
     for _ in range(3):
         log.warning(msg)
@@ -243,3 +239,17 @@ def clean(token, none=False):
         return None
     else:
         return token.strip().lower()
+
+
+def insert(generator, index, extra):
+    for i, value in enumerate(generator):
+        if i == index:
+            yield extra
+        yield value
+
+
+def str_or_none(value):
+    if value is None:
+        return value
+    else:
+        return str(value)
