@@ -5,8 +5,8 @@ from .elevation import ElevationCalculator
 from .heart_rate import RestHRCalculator
 from .impulse import ImpulseCalculator
 from .kit import KitCalculator
-from .monitor import MonitorCalculator
+from .steps import StepsCalculator
 from .nearby import NearbyCalculator
 from .response import ResponseCalculator
-from .segment import SegmentCalculator
 from .summary import SummaryCalculator
+from .sector import SectorCalculator
