@@ -1,11 +1,8 @@
 
 from logging import getLogger
 
-from sqlalchemy.sql.functions import count
-
 from .utils import Displayer
 from ...diary.model import value, optional_text, text
-from ...global_ import global_data
 from ...sql import Interval
 
 log = getLogger(__name__)
@@ -15,12 +12,9 @@ class DatabaseDisplayer(Displayer):
 
     @optional_text('Database')
     def _read_date(self, s, date):
-        ids = set(d.interval_id for d in global_data().sys.get_dirty_intervals())
-        total = s.query(count(Interval.id)).filter(Interval.id.in_(ids)).count()
-        today = s.query(count(Interval.id)). \
-            filter(Interval.id.in_(ids),
-                   Interval.start <= date,
-                   Interval.finish > date).scalar()
+        q = s.query(Interval.id).filter(Interval.dirty == True, Interval.permanent == False)
+        total = q.count()
+        today = q.filter(Interval.start <= date, Interval.finish > date).count()
         if not total:
             yield text('No dirty statistics')
         else:
