@@ -1,14 +1,15 @@
 
 from .achievement import Achievement
 from .activity import ActivityGroup, ActivityTimespan, ActivityJournal, ActivityBookmark
+from .cluster import ClusterInputScratch, ClusterHull, ClusterFragmentScratch
 from .constant import Constant
 from .file import FileScan, FileHash
 from .kit import KitGroup, KitItem, KitComponent, KitModel
 from .monitor import MonitorJournal
 from .nearby import ActivitySimilarity, ActivityNearby
 from .pipeline import Pipeline, PipelineType
-from .segment import Segment, SegmentJournal
-from .source import Source, Interval, NoStatistics, Dummy, Composite, CompositeComponent
+from .sector import SectorGroup, Sector, SectorClimb, SectorJournal, SectorType
+from .source import Source, Interval, NoStatistics, Composite, CompositeComponent
 from .statistic import StatisticName, StatisticJournalFloat, StatisticJournalText, StatisticJournalInteger, \
     StatisticJournalTimestamp, StatisticJournal, StatisticMeasure, StatisticJournalType
 from .system import SystemConstant, Process
