@@ -6,30 +6,32 @@ from sqlalchemy import Column, Integer, ForeignKey, desc, or_
 from sqlalchemy.orm import relationship, aliased, backref
 from sqlalchemy.orm.exc import NoResultFound
 
-from .source import SourceType, Composite, CompositeComponent, UngroupedSource
+from .source import SourceType, Composite, CompositeComponent, UngroupedSource, Source
 from .statistic import StatisticJournal, StatisticName, StatisticJournalTimestamp
 from ..support import Base
+from ..triggers import add_child_ddl
 from ..types import Name
 from ..utils import add
-from ...commands.args import FORCE, mm
+from ...commands.args import FORCE
+from ...common.args import mm
+from ...common.date import YMD
 from ...diary.model import TYPE, DB, UNITS
 from ...lib import now, time_to_local_time
-from ...lib.date import YMD
 from ...lib.utils import inside_interval
-from ...names import Titles, Names, Units
+from ...names import T, N, U
 
 log = getLogger(__name__)
 
 NAME = 'name'
 GROUP = 'group'
 ITEM = 'item'
-ITEMS = Names._s(ITEM)
+ITEMS = N._s(ITEM)
 COMPONENT = 'component'
-COMPONENTS = Names._s(COMPONENT)
+COMPONENTS = N._s(COMPONENT)
 MODEL = 'model'
-MODELS = Names._s(MODEL)
+MODELS = N._s(MODEL)
 STATISTICS = 'statistics'
-N = 'n'
+_N = 'n'
 SUM = 'sum'
 MEAN = 'mean'
 MEDIAN = 'median'
@@ -104,42 +106,37 @@ class StatisticsMixin:
     support for reading and writing statistics related to a kit class.
     '''
 
-    def _base_statistic_query(self, s, statistic, *sources, owner=None):
-        sources = (self,) + sources
-        subq = s.query(Composite.id.label('composite_id'))
-        for source in sources:
-            cc = aliased(CompositeComponent)
-            subq = subq.join(cc, Composite.id == cc.output_source_id).filter(cc.input_source == source)
-        subq = subq.subquery()
-        q = s.query(StatisticJournal). \
-            join(StatisticName). \
-            outerjoin(subq, subq.c.composite_id == StatisticJournal.source_id). \
-            filter(StatisticName.name == statistic)
-        if len(sources) == 1:
-            q = q.filter(or_(StatisticJournal.source == self, subq.c.composite_id != None))
+    def _base_statistic_query(self, s, statistic, owner=None, direct=False):
+        # direct should only be true for statistics we know are set directly (added, removed, etc, NOT used)
+        q = s.query(StatisticJournal).join(StatisticName).filter(StatisticName.name == statistic)
+        if direct:
+            q = q.filter(StatisticJournal.source == self)
         else:
-            q = q.filter(subq.c.composite_id != None)
+            via_composite = s.query(Composite.id). \
+                join(CompositeComponent, CompositeComponent.output_source_id == Composite.id). \
+                filter(CompositeComponent.input_source == self)
+            q = q.filter(or_(StatisticJournal.source == self,
+                             StatisticJournal.source_id.in_(via_composite)))
         if owner:
             q = q.filter(StatisticName.owner == owner)
         return q
 
     def _base_use_query(self, s, statistic):
         cc1, cc2 = aliased(CompositeComponent), aliased(CompositeComponent)
-        sourceq = s.query(cc1.input_source_id). \
-            join(Composite, Composite.id == cc1.output_source_id). \
-            join(cc2, cc2.output_source_id == Composite.id). \
-            filter(cc2.input_source == self,
-                   cc1.input_source != self).subquery()
-        return s.query(StatisticJournal). \
+        query = s.query(StatisticJournal). \
             join(StatisticName). \
-            filter(StatisticName.name == statistic,
-                   StatisticJournal.source_id.in_(sourceq))
+            filter(StatisticName.name == statistic). \
+            join(cc1, StatisticJournal.source_id == cc1.input_source_id). \
+            join(Composite, Composite.id == cc1.output_source_id). \
+            join(cc2, Composite.id == cc2.output_source_id). \
+            filter(cc2.input_source == self)
+        return query
 
-    def _get_statistic(self, s, statistic, *sources, owner=None):
-        return self._base_statistic_query(s, statistic, *sources, owner=owner).one()
+    def _get_statistic(self, s, statistic, owner=None, direct=False):
+        return self._base_statistic_query(s, statistic, owner=owner, direct=direct).one()
 
-    def _get_statistics(self, s, statistic, *sources, owner=None):
-        return self._base_statistic_query(s, statistic, *sources, owner=owner).all()
+    def _get_statistics(self, s, statistic, owner=None, direct=False):
+        return self._base_statistic_query(s, statistic, owner=owner, direct=direct).all()
 
     def _remove_statistic(self, s, statistic, *sources, owner=None):
         # cannot delete directly with join
@@ -155,26 +152,26 @@ class StatisticsMixin:
         else:
             source = self
         owner = owner or self
-        return StatisticJournalTimestamp.add(s, statistic, None, None, owner, source, time,
+        return StatisticJournalTimestamp.add(s, statistic, None, None, owner, source, time, time,
                                              description='A timestamp for tracking kit use.')
 
     def time_added(self, s):
-        return self._get_statistic(s, Names.KIT_ADDED).time
+        return self._get_statistic(s, N.KIT_ADDED, direct=True).time
 
     def time_expired(self, s):
         try:
-            return self._get_statistic(s, Names.KIT_RETIRED).time
+            return self._get_statistic(s, N.KIT_RETIRED, direct=True).time
         except NoResultFound:
             return None
 
     def add_use(self, s, time, source=None, owner=None):
-        self._add_timestamp(s, Titles.KIT_USED, time, source=source, owner=owner)
+        self._add_timestamp(s, T.KIT_USED, time, source=source, owner=owner)
 
     def active_times(self, s):
-        return self._base_use_query(s, Names.ACTIVE_TIME).all()
+        return self._base_use_query(s, N.ACTIVE_TIME).all()
 
     def active_distances(self, s):
-        return self._base_use_query(s, Names.ACTIVE_DISTANCE).all()
+        return self._base_use_query(s, N.ACTIVE_DISTANCE).all()
 
     def lifetime(self, s):
         added, expired = self.time_added(s), self.time_expired(s)
@@ -183,10 +180,10 @@ class StatisticsMixin:
 
     def _add_individual_statistics(self, s, model):
         model_statistics = []
-        self._calculate_individual_statistics(model_statistics, Titles.ACTIVE_DISTANCE, self.active_distances(s), Units.KM)
-        self._calculate_individual_statistics(model_statistics, Titles.ACTIVE_TIME, self.active_times(s), Units.S)
+        self._calculate_individual_statistics(model_statistics, T.ACTIVE_DISTANCE, self.active_distances(s), U.KM)
+        self._calculate_individual_statistics(model_statistics, T.ACTIVE_TIME, self.active_times(s), U.S)
         expire = self.time_expired(s) or now()
-        model_statistics.append({NAME: Titles.AGE, N: 1, SUM: (expire - self.time_added(s)).days, UNITS: Units.D})
+        model_statistics.append({NAME: T.AGE, _N: 1, SUM: (expire - self.time_added(s)).days, UNITS: U.D})
         model[STATISTICS] = model_statistics
 
     def _calculate_individual_statistics(self, model_statistics, name, values, units):
@@ -195,7 +192,7 @@ class StatisticsMixin:
             values = [value.value for value in values]
             total = sum(values)
             # had mean and median, but they were pointless
-            model_statistics.append({NAME: name, N: n, SUM: total, UNITS: units})
+            model_statistics.append({NAME: name, _N: n, SUM: total, UNITS: units})
 
 
 class ModelMixin:
@@ -265,6 +262,7 @@ class KitGroup(ModelMixin, Base):
         return f'KitGroup "{self.name}"'
 
 
+@add_child_ddl(Source)
 class KitItem(ModelMixin, StatisticsMixin, UngroupedSource):
     '''
     an individual kit item (a particular bike, a particular shoe, etc)
@@ -294,7 +292,7 @@ class KitItem(ModelMixin, StatisticsMixin, UngroupedSource):
             return item
 
     def _add_statistics(self, s, time):
-        self._add_timestamp(s, Titles.KIT_ADDED, time)
+        self._add_timestamp(s, T.KIT_ADDED, time)
 
     @classmethod
     def get(cls, s, name):
@@ -313,10 +311,10 @@ class KitItem(ModelMixin, StatisticsMixin, UngroupedSource):
     def finish(self, s, date, force):
         if self.time_expired(s):
             if force:
-                self._remove_statistic(s, Names.KIT_RETIRED, owner=self)
+                self._remove_statistic(s, N.KIT_RETIRED, owner=self)
             else:
                 raise Exception(f'Item {self.name} is already retired')
-        self._add_timestamp(s, Titles.KIT_RETIRED, date)
+        self._add_timestamp(s, T.KIT_RETIRED, date)
 
     def delete(self, s):
         s.delete(self)
@@ -432,6 +430,7 @@ class KitComponent(ModelMixin, Base):
         return f'KitComponent "{self.name}"'
 
 
+@add_child_ddl(Source)
 class KitModel(ModelMixin, StatisticsMixin, UngroupedSource):
     '''
     a particular piece of a kit item (a particular bike wheel, a particular set of laces, etc).
@@ -465,7 +464,7 @@ class KitModel(ModelMixin, StatisticsMixin, UngroupedSource):
         if s.query(StatisticJournal). \
                 join(StatisticName). \
                 join(KitModel, KitModel.id == StatisticJournal.source_id). \
-                filter(StatisticName.name == Names.KIT_ADDED,
+                filter(StatisticName.name == N.KIT_ADDED,
                        StatisticJournal.time == time,
                        KitModel.name == name,
                        KitModel.component == component,
@@ -481,30 +480,30 @@ class KitModel(ModelMixin, StatisticsMixin, UngroupedSource):
         return add(s, KitModel(item=item, component=component, name=name))
 
     def _add_statistics(self, s, time):
-        self._add_timestamp(s, Titles.KIT_ADDED, time)
+        self._add_timestamp(s, T.KIT_ADDED, time)
         before = self.before(s, time)
         after = self.after(s, time)
         if before:
             before_expiry = before.time_expired(s)
             if before_expiry and before_expiry > time:
-                before._remove_statistic(s, Names.KIT_RETIRED)
+                before._remove_statistic(s, N.KIT_RETIRED)
                 before_expiry = None
             if not before_expiry:
-                before._add_timestamp(s, Titles.KIT_RETIRED, time)
+                before._add_timestamp(s, T.KIT_RETIRED, time)
                 log.info(f'Retired previous {self.component.name} ({before.name})')
         if after:
             after_added = after.time_added(s)
-            self._add_timestamp(s, Titles.KIT_RETIRED, after_added)
+            self._add_timestamp(s, T.KIT_RETIRED, after_added)
             log.info(f'Retired new {self.component.name} ({self.name})')
 
     @classmethod
     def get_all_at(cls, s, item, time):
         beforeq = s.query(StatisticJournalTimestamp.source_id, StatisticJournalTimestamp.time). \
             join(StatisticName). \
-            filter(StatisticName.name == Names.KIT_ADDED).subquery()
+            filter(StatisticName.name == N.KIT_ADDED).subquery()
         afterq = s.query(StatisticJournalTimestamp.source_id, StatisticJournalTimestamp.time). \
             join(StatisticName). \
-            filter(StatisticName.name == Names.KIT_RETIRED).subquery()
+            filter(StatisticName.name == N.KIT_RETIRED).subquery()
         return s.query(KitModel). \
             join(beforeq, beforeq.c.source_id == KitModel.id). \
             outerjoin(afterq, afterq.c.source_id == KitModel.id). \
@@ -525,10 +524,10 @@ class KitModel(ModelMixin, StatisticsMixin, UngroupedSource):
         if time:
             beforeq = s.query(StatisticJournalTimestamp.source_id, StatisticJournalTimestamp.time). \
                 join(StatisticName). \
-                filter(StatisticName.name == Names.KIT_ADDED).subquery()
+                filter(StatisticName.name == N.KIT_ADDED).subquery()
             afterq = s.query(StatisticJournalTimestamp.source_id, StatisticJournalTimestamp.time). \
                 join(StatisticName). \
-                filter(StatisticName.name == Names.KIT_RETIRED).subquery()
+                filter(StatisticName.name == N.KIT_RETIRED).subquery()
             q = q.join(beforeq, beforeq.c.source_id == KitModel.id). \
                 outerjoin(afterq, afterq.c.source_id == KitModel.id). \
                 filter(beforeq.c.time <= time,
@@ -553,13 +552,13 @@ class KitModel(ModelMixin, StatisticsMixin, UngroupedSource):
     def before(self, s, time=None):
         if not time:
             time = self.time_added(s)
-        return self._base_sibling_query(s, Names.KIT_ADDED).filter(StatisticJournal.time < time). \
+        return self._base_sibling_query(s, N.KIT_ADDED).filter(StatisticJournal.time < time). \
             order_by(desc(StatisticJournal.time)).first()
 
     def after(self, s, time=None):
         if not time:
             time = self.time_added(s)
-        return self._base_sibling_query(s, Names.KIT_ADDED).filter(StatisticJournal.time > time). \
+        return self._base_sibling_query(s, N.KIT_ADDED).filter(StatisticJournal.time > time). \
             order_by(StatisticJournal.time).first()
 
     def undo(self, s):
@@ -567,10 +566,10 @@ class KitModel(ModelMixin, StatisticsMixin, UngroupedSource):
         s.delete(self)
         before = self.before(s, time)
         if before:
-            before._remove_statistic(s, Names.KIT_RETIRED)
+            before._remove_statistic(s, N.KIT_RETIRED)
             after = self.after(s, time)
             if after:
-                before._add_timestamp(s, Titles.KIT_RETIRED, after.time_added(s))
+                before._add_timestamp(s, T.KIT_RETIRED, after.time_added(s))
 
     def time_range(self, s):
         return None, None
