@@ -3,19 +3,21 @@ import datetime as dt
 from enum import IntEnum
 from logging import getLogger
 
-from sqlalchemy import Column, Integer, ForeignKey, Text, UniqueConstraint, Float, desc, asc, Index
+from geoalchemy2 import Geography
+from sqlalchemy import Column, Integer, ForeignKey, Text, UniqueConstraint, Float, desc, asc, Index, DateTime
 from sqlalchemy.exc import IntegrityError
-from sqlalchemy.orm import relationship, backref, synonym
+from sqlalchemy.orm import relationship, backref
 from sqlalchemy.orm.exc import NoResultFound
 
 from .source import Interval
 from ..support import Base
-from ..types import Time, ShortCls, Name, name_and_title
-from ..utils import add
+from ..triggers import add_child_ddl, add_text
+from ..types import ShortCls, Name, name_and_title, Point, UTC
+from ..utils import add, WGS84_SRID
+from ...common.date import format_seconds, local_date_to_time, time_to_local_time
 from ...diary.model import TYPE, MEASURES, SCHEDULES
-from ...lib.date import format_seconds, local_date_to_time, time_to_local_time
 from ...lib.utils import sigfig
-from ...names import Units, simple_name
+from ...names import U, simple_name
 
 log = getLogger(__name__)
 
@@ -34,7 +36,7 @@ class StatisticName(Base):
     description = Column(Text)
     units = Column(Text)
     summary = Column(Text)  # max, min, etc - comma separated list
-    owner = Column(ShortCls, nullable=False, index=True)  # index for deletion
+    owner = Column(ShortCls, nullable=False, index=True)  # index for deletion + exclusion in intervals
     statistic_journal_type = Column(Integer, nullable=False)  # StatisticJournalType
     UniqueConstraint(name, owner)
 
@@ -61,12 +63,12 @@ class StatisticName(Base):
 
     @classmethod
     def add_if_missing(cls, s, name, type, units, summary, owner, description=None, title=None):
-        s.commit()  # start new transaction here in case rollback
         q = s.query(StatisticName). \
             filter(StatisticName.name == name,
                    StatisticName.owner == owner)
         statistic_name = q.one_or_none()
         if not statistic_name:
+            s.commit()  # start new transaction here in case rollback
             statistic_name = add(s, StatisticName(name=name, units=units, summary=summary, owner=owner,
                                                   statistic_journal_type=type, description=description,
                                                   title=title))
@@ -144,20 +146,18 @@ class StatisticJournal(Base):
 
     id = Column(Integer, primary_key=True)
     type = Column(Integer, nullable=False, index=True)  # index needed for fast delete of subtypes
-    statistic_name_id = Column(Integer, ForeignKey('statistic_name.id', ondelete='cascade'),
-                               nullable=False, index=True)
+    statistic_name_id = Column(Integer, ForeignKey('statistic_name.id', ondelete='cascade'), nullable=False)
     statistic_name = relationship('StatisticName')
-    source_id = Column(Integer, ForeignKey('source.id', ondelete='cascade'),
-                       nullable=False, index=True)
+    source_id = Column(Integer, ForeignKey('source.id', ondelete='cascade'), nullable=False)
     source = relationship('Source')
-    time = Column(Time, nullable=False)
+    time = Column(UTC, nullable=False, index=True)
     # serial "counts" along values in the timeseries.  it's optional.  for garmin, all values appear each
     # record, so all imported values share the same serial.  but that's not true for the corrected elevation,
     # for example.
     serial = Column(Integer)
 
-    UniqueConstraint(statistic_name_id, time, source_id)
-    UniqueConstraint(serial, source_id, statistic_name_id)
+    UniqueConstraint(statistic_name_id, time, source_id)  # helps with latest value for name
+    UniqueConstraint(statistic_name_id, source_id, serial)
     Index('from_activity_timespan', source_id, statistic_name_id, time)  # time last since inequality
 
     __mapper_args__ = {
@@ -200,16 +200,16 @@ class StatisticJournal(Base):
         units = self.statistic_name.units
         if not units:
             return '%d' % self.value
-        elif units == Units.M:
+        elif units == U.M:
             if self.value > 2000:
                 return '%d km' % (self.value / 1000)
             else:
                 return '%d m' % self.value
-        elif units == Units.S:
+        elif units == U.S:
             return format_seconds(self.value)
-        elif units in (Units.KMH, Units.PC, Units.BPM, Units.STEPS_UNITS, Units.W, Units.KJ):
+        elif units in (U.KMH, U.PC, U.BPM, U.STEPS_UNITS, U.W, U.KJ):
             return '%d %s' % (self.value, units)
-        elif units == Units.KCAL:
+        elif units == U.KCAL:
             return '%s %s' % (sigfig(self.value, 2), units)
         else:
             return '%d %s' % (self.value, units)
@@ -231,10 +231,12 @@ class StatisticJournal(Base):
     def measures_as_model(self, date):
         if hasattr(self, 'measures'):
             measures = {TYPE: MEASURES, SCHEDULES: {}}
+            # order by day, month, year etc
             for measure in sorted(self.measures,
                                   key=lambda measure: measure.source.schedule.frame_length_in_days(date),
                                   reverse=True):
-                measures[SCHEDULES][measure.source.schedule.describe(compact=True)] = (measure.percentile, measure.rank)
+                measures[SCHEDULES][measure.source.schedule.describe(compact=True)] = \
+                    (measure.percentile, measure.rank)
             return measures
         else:
             return None
@@ -244,7 +246,7 @@ class StatisticJournal(Base):
         from . import ActivityGroup, Source
         start = local_date_to_time(date)
         finish = start + dt.timedelta(days=1)
-        q = s.query(StatisticJournal).join(StatisticName, Source). \
+        q = s.query(StatisticJournal).join(StatisticName).join(Source). \
             filter(StatisticName.name == name,
                    StatisticJournal.time >= start,
                    StatisticJournal.time < finish,
@@ -257,7 +259,7 @@ class StatisticJournal(Base):
     @classmethod
     def at(cls, s, time, name, owner, activity_group):
         from . import ActivityGroup, Source
-        return s.query(StatisticJournal).join(StatisticName, Source). \
+        return s.query(StatisticJournal).join(StatisticName).join(Source). \
             filter(StatisticName.name == name,
                    StatisticJournal.time == time,
                    StatisticName.owner == owner,
@@ -266,7 +268,7 @@ class StatisticJournal(Base):
     @classmethod
     def at_like(cls, s, time, name, owner, activity_group):
         from . import ActivityGroup, Source
-        return s.query(StatisticJournal).join(StatisticName, Source). \
+        return s.query(StatisticJournal).join(StatisticName).join(Source). \
             filter(StatisticName.name.like(name),
                    StatisticJournal.time == time,
                    StatisticName.owner == owner,
@@ -275,7 +277,7 @@ class StatisticJournal(Base):
     @classmethod
     def before(cls, s, time, name, owner, activity_group):
         from . import ActivityGroup, Source
-        return s.query(StatisticJournal).join(StatisticName, Source). \
+        return s.query(StatisticJournal).join(StatisticName).join(Source). \
             filter(StatisticName.name == name,
                    StatisticJournal.time <= time,
                    StatisticName.owner == owner,
@@ -303,7 +305,7 @@ class StatisticJournal(Base):
     @classmethod
     def after(cls, s, time, name, owner, activity_group):
         from . import ActivityGroup, Source
-        return s.query(StatisticJournal).join(StatisticName, Source). \
+        return s.query(StatisticJournal).join(StatisticName).join(Source). \
             filter(StatisticName.name == name,
                    StatisticJournal.time >= time,
                    StatisticName.owner == owner,
@@ -312,7 +314,7 @@ class StatisticJournal(Base):
 
     @classmethod
     def at_interval(cls, s, start, schedule, statistic_owner, statistic_name, interval_owner):
-        return s.query(StatisticJournal).join(StatisticName, Interval). \
+        return s.query(StatisticJournal).join(StatisticName).join(Interval). \
                     filter(StatisticJournal.statistic_name_id == StatisticName.id,
                            Interval.schedule == schedule,
                            Interval.start == start,
@@ -325,13 +327,14 @@ class StatisticJournal(Base):
     @classmethod
     def for_source(cls, s, source_id, name, owner, activity_group):
         from . import ActivityGroup, Source
-        return s.query(StatisticJournal).join(StatisticName, Source). \
+        return s.query(StatisticJournal).join(StatisticName).join(Source). \
             filter(StatisticName.name == name,
                    StatisticName.owner == owner,
                    Source.activity_group == ActivityGroup.from_name(s, activity_group),
                    StatisticJournal.source_id == source_id).one_or_none()
 
 
+@add_child_ddl(StatisticJournal)
 class StatisticJournalInteger(StatisticJournal):
 
     __tablename__ = 'statistic_journal_integer'
@@ -343,15 +346,13 @@ class StatisticJournalInteger(StatisticJournal):
         'polymorphic_identity': StatisticJournalType.INTEGER
     }
 
-    def set(self, value):
-        self.value = value if value is None else int(value)
-
     @classmethod
     def add(cls, s, name, units, summary, owner, source, value, time, serial=None, description=None):
         return super().add(s, name, units, summary, owner, source, value, time, serial,
                            StatisticJournalType.INTEGER, description=description)
 
 
+@add_child_ddl(StatisticJournal)
 class StatisticJournalFloat(StatisticJournal):
 
     __tablename__ = 'statistic_journal_float'
@@ -359,9 +360,6 @@ class StatisticJournalFloat(StatisticJournal):
     id = Column(Integer, ForeignKey('statistic_journal.id', ondelete='cascade'), primary_key=True)
     value = Column(Float, nullable=False)
 
-    def set(self, value):
-        self.value = value if value is None else float(value)
-
     @classmethod
     def add(cls, s, name, units, summary, owner, source, value, time, serial=None, description=None):
         return super().add(s, name, units, summary, owner, source, value, time, serial,
@@ -377,30 +375,31 @@ class StatisticJournalFloat(StatisticJournal):
         units = self.statistic_name.units
         if not units:
             return '%f' % self.value
-        elif units == Units.M:
+        elif units == U.M:
             if self.value > 2000:
                 return '%.1f km' % (self.value / 1000)
             else:
                 return '%d m' % int(self.value)
-        elif units == Units.KM:
+        elif units == U.KM:
             if self.value > 2:
                 return '%.1f km' % self.value
             else:
                 return '%d m' % int(self.value * 1000)
-        elif units == Units.S:
+        elif units == U.S:
             return format_seconds(self.value)
-        elif units in (Units.KMH, Units.PC, Units.KG, Units.W, Units.KJ):
+        elif units in (U.KMH, U.PC, U.KG, U.W, U.KJ):
             return '%.1f %s' % (self.value, units)
-        elif units == Units.KCAL:
+        elif units == U.KCAL:
             return '%s %s' % (sigfig(self.value, 2), units)
-        elif units in (Units.BPM, Units.STEPS_UNITS):
+        elif units in (U.BPM, U.STEPS_UNITS):
             return '%d %s' % (int(self.value), units)
-        elif units == Units.FF:
+        elif units == U.FF:
             return '%d' % int(self.value)
         else:
             return '%s %s' % (self.value, units)
 
 
+@add_child_ddl(StatisticJournal)
 class StatisticJournalText(StatisticJournal):
 
     __tablename__ = 'statistic_journal_text'
@@ -408,9 +407,6 @@ class StatisticJournalText(StatisticJournal):
     id = Column(Integer, ForeignKey('statistic_journal.id', ondelete='cascade'), primary_key=True)
     value = Column(Text, nullable=False)
 
-    def set(self, value):
-        self.value = value if value is None else str(value)
-
     @classmethod
     def add(cls, s, name, units, summary, owner, source, value, time, serial=None, description=None):
         return super().add(s, name, units, summary, owner, source, value, time, serial,
@@ -430,24 +426,22 @@ class StatisticJournalText(StatisticJournal):
             return '%s %s' % (self.value, units)
 
 
+@add_child_ddl(StatisticJournal)
 class StatisticJournalTimestamp(StatisticJournal):
 
     __tablename__ = 'statistic_journal_timestamp'
 
     id = Column(Integer, ForeignKey('statistic_journal.id', ondelete='cascade'), primary_key=True)
-    value = synonym('time')
+    value = Column(UTC, nullable=False)
 
     __mapper_args__ = {
         'polymorphic_identity': StatisticJournalType.TIMESTAMP
     }
 
     @classmethod
-    def add(cls, s, name, units, summary, owner, source, time, serial=None, description=None):
-        statistic_name = StatisticName.add_if_missing(s, name, StatisticJournalType.TIMESTAMP,
-                                                      units, summary, owner, description=description)
-        journal = StatisticJournalTimestamp(statistic_name=statistic_name, source=source, time=time, serial=serial)
-        s.add(journal)
-        return journal
+    def add(cls, s, name, units, summary, owner, source, value, time, serial=None, description=None):
+        return super().add(s, name, units, summary, owner, source, value, time, serial,
+                           StatisticJournalType.TIMESTAMP, description=description)
 
     def formatted(self):
         return time_to_local_time(self.value)
